@@ -544,8 +544,14 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 	MouseEvent *mev;
 	TextDEF *LabelDef;
 	GraphObj **tmpPlots;
+	AxisDEF *axis;
 
 	switch(cmd){
+	case CMD_SET_AXDEF:
+		if(axis = (AxisDEF*)tmpl) {
+			flags = (flags & AXIS_MINORTICK) | axis->flags;
+			}
+		break;
 	case CMD_FLUSH:
 		if(Grid) DeleteGO(Grid);		Grid = 0L;
 		if(label) DeleteGO(label);		label = 0L;
@@ -620,11 +626,7 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 		case MOUSE_LBUP:
 			if(IsInRect(&rDims, mev->x, mev->y) && !CurrGO && 
 				IsCloseToLine(&pts[0], &pts[1], mev->x, mev->y)) {
-				if(pts[0].x == pts[1].x || pts[0].y == pts[1].y) {
-					o->ShowMark(&rDims, MRK_INVERT);
-					CurrGO = this;
-					}
-				else o->ShowMark(this, MRK_GODRAW);
+				o->ShowMark(this, MRK_GODRAW);
 				return true;
 				}
 			break;
@@ -1262,11 +1264,7 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 				}
 			else if(IsInRect(&rDims, mev->x, mev->y) && !CurrGO && 
 				IsCloseToLine(&pts[0], &pts[1], mev->x, mev->y)) {
-				if(pts[0].x == pts[1].x || pts[0].y == pts[1].y){ 
-					o->ShowMark(&rDims, MRK_INVERT);
-					CurrGO = this;
-					}
-				else o->ShowMark(this, MRK_GODRAW);
+				o->ShowMark(this, MRK_GODRAW);
 				return true;
 				}
 			break;
@@ -1276,14 +1274,25 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 			if(Ticks[i] && Ticks[i]->Command(cmd, tmpl, o)) return true;
 		return false;
 	case CMD_SET_AXDEF:
-		if(tmpl) {
+		if(axis = (AxisDEF*)tmpl) {
 			if(axis && axis->owner == (void*)this) {
 				if(axis->breaks) free(axis->breaks);
 				free(axis);
 				}
+			if(Ticks) for(i = 0; i < NumTicks; i++)
+				if(Ticks[i]) {
+					Ticks[i]->Command(cmd, tmpl, o);
+					Ticks[i]->Command(CMD_SET_GRIDTYPE, (void*) &gl_type, 0L);
+					Ticks[i]->Command(CMD_TLB_TXTDEF, &tlbdef, 0L);
+					if(axis->flags & AXIS_GRIDLINE) Ticks[i]->Command(CMD_SET_GRIDLINE, &GridLine, 0L);
+					Ticks[i]->SetSize(SIZE_TICK_ANGLE, tick_angle);
+					Ticks[i]->SetSize(SIZE_AXIS_TICKS, sizAxTick);
+					Ticks[i]->SetSize(SIZE_LB_XDIST, tlbdist.fx);
+					Ticks[i]->SetSize(SIZE_LB_YDIST, tlbdist.fy);
+					Ticks[i]->Command(CMD_TICK_TYPE, &tick_type, 0L);
+					}
+			return true;
 			}
-		axis = (AxisDEF*)tmpl;
-		if(axis) return true;
 		return false;
 	case CMD_CAN_DELETE:
 		if(axis->owner == (void*)this) return true;
@@ -1297,18 +1306,16 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 	case CMD_DELOBJ:
 		o->HideMark();
 		if(!tmpl || !parent) return false;
-		if(Ticks && NumTicks) for(i = 0; i < NumTicks; i++){ 
-			if(Ticks[i] == tmpl) {
-				Undo.DeleteGO((GraphObj**)(&Ticks[i]), 0L, o);
-				return parent->Command(CMD_REDRAW, 0L, o);
-				}
-			else if(Ticks[i] && Ticks[i]->Command(cmd, tmpl, o))
-				return parent->Command(CMD_REDRAW, 0L, o);
-			}
+		if(DeleteGOL((GraphObj***) &Ticks, NumTicks, (GraphObj *)tmpl, o)) 
+			return parent->Command(CMD_REDRAW, 0L, o);
 		if(tmpl == (void*)axisLabel) {
 			Undo.DeleteGO((GraphObj**)(&axisLabel), 0L, o);
 			return parent->Command(CMD_REDRAW, 0L, o);
 			}
+		if(Ticks && NumTicks) for(i = 0; i < NumTicks; i++){ 
+			if(Ticks[i] && Ticks[i]->Command(cmd, tmpl, o))
+				return parent->Command(CMD_REDRAW, 0L, o);
+			}
 		break;
 	case CMD_MUTATE:
 		if(!parent || !(tmpPlots = (GraphObj **)tmpl) || !tmpPlots[0] || !tmpPlots[1]) return false;
