@@ -799,14 +799,26 @@ ExportSVG::oSolidLine(POINT *p)
 bool
 ExportSVG::oTextOut(int x, int y, char *txt, int cb)
 {
-	int h, ix, iy;
+	int i, h, ix, iy;
 	char tmptxt[140], *nt;
 
-	h = TxtSet.iSize;
+	if(!txt || !txt[0]) return false;
+	else h = TxtSet.iSize;
 	if(TxtSet.Align & TXA_VCENTER) iy = y + h/3;
 	else if(TxtSet.Align & TXA_VBOTTOM) iy = y;
-	else iy = y + iround(h*.8f);
+	else iy = y + iround(h * 0.8);
 	ix = x;
+	if((TxtSet.Style & TXS_SUB) || (TxtSet.Style & TXS_SUPER)) h = iround((double)TxtSet.iSize * 0.8);
+	if(TxtSet.Style & TXS_SUB) {
+		if((TxtSet.Align & TXA_VCENTER) == TXA_VCENTER) iy +=un2iy(TxtSet.fSize*0.4);
+		else if((TxtSet.Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy += un2iy(TxtSet.fSize*0.2);
+		else if((TxtSet.Align & TXA_VTOP) == TXA_VTOP) iy += un2iy(TxtSet.fSize*.6);
+		}
+	else if(TxtSet.Style & TXS_SUPER) {
+		if((TxtSet.Align & TXA_VCENTER) == TXA_VCENTER) iy -= un2iy(TxtSet.fSize*0.4);
+		else if((TxtSet.Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy -= un2iy(TxtSet.fSize*0.6);
+		else if((TxtSet.Align & TXA_VTOP) == TXA_VTOP) iy -= un2iy(TxtSet.fSize*0.2);
+		}
 	sprintf(output, "<text x=\"%d\" y=\"%d\" ", ix, iy);
 	if(fabs(TxtSet.RotBL) >.01 || fabs(TxtSet.RotCHAR) >.01) {
 		sprintf(tmptxt,"transform=\"rotate(%.0f,%d,%d)\" ", -TxtSet.RotBL, ix, iy);
@@ -825,10 +837,17 @@ ExportSVG::oTextOut(int x, int y, char *txt, int cb)
 	sprintf(tmptxt, " fill:%s; stroke:%s; ", ColName(TxtSet.ColTxt),
 		ColName(TxtSet.ColTxt));
 	AddToOutput(tmptxt);
-	sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", TxtSet.iSize, 
+	sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", h, 
 		(TxtSet.Align & TXA_HRIGHT) ? "end" : (TxtSet.Align & TXA_HCENTER) ? "middle":"start");
 	AddToOutput(tmptxt);
 	nt = str2xml(txt);
+	if((TxtSet.Style & TXS_SUB) || (TxtSet.Style & TXS_SUPER)) {
+		for(i = strlen(nt)+6, nt[i+1]=0; i > 5; i--) {
+			nt[i] = nt[i-6];
+			}
+		nt[0] = '&';		nt[1] = 'n';		nt[2] = 'b';
+		nt[3] = 's';		nt[4] = 'p';		nt[5] = ';';
+		}
 	if((strlen(indent)+strlen(nt)+strlen(output)) <110) strcat(output, nt);
 	else {
 		fprintf(oFile, "%s%s\n", indent, output);
