@@ -38,6 +38,7 @@ extern GraphObj *CurrGO;			//Selected Graphic Objects
 extern Default defs;
 extern int dlgtxtheight;
 extern char TmpTxt[];
+extern int cPlots;
 
 static notary *Notary = 0L;
 static ReadCache *Cache = 0L;
@@ -81,8 +82,14 @@ int OpenOutputFile(char *file)
 		if(!ptr) goto openerr;
 		ti = time(0L);
 		cbOut = sprintf(ptr,";RLP 1.0\n;File \"%s\" created by RLPlot version "
-			""SZ_VERSION"\n;Date/Time: %s",
-			file, ctime(&ti));
+			""SZ_VERSION" for %s \n;Date/Time: %s ",
+			file,  
+#ifdef _WINDOWS
+			"Windows",
+#else
+			"Qt",
+#endif
+			ctime(&ti));
 		}
 	return iFile;
 openerr:
@@ -196,6 +203,32 @@ void WriteTypFpLst3D(fPOINT3D *ptr, long count)
 		}
 }
 
+static char * esc_str = 0L;
+static int esc_str_size = 0;
+int WriteEscString(char *txt, char *buff, int cb)
+{
+	int i, j, l, lim = 60;
+
+	if(!txt || !txt[0]) return 0;
+	l = strlen(txt);
+	if((l+10) > esc_str_size) esc_str = (char*)realloc(esc_str, esc_str_size = (l+10));
+	j = 0;	esc_str[j++] = '"';
+	for(i = 0; txt[i]; i++) {
+		switch(txt[i]) {
+		case '\\':	j += sprintf(esc_str+j, "\\\\");	break;
+		case '\n':	j += sprintf(esc_str+j, "\\n");		break;
+		default:	esc_str[j++] = txt[i];
+			}
+		if(j > esc_str_size -2) esc_str = (char*)realloc(esc_str, (esc_str_size += 20));
+		if(j > lim) {
+			j += sprintf(esc_str+j, "\"\n   \"");
+			lim += 60;
+			}
+		}
+	j += sprintf(esc_str+j, "\"\n");
+	return sprintf(buff+cb, "%s", esc_str);
+}
+
 bool ExecOutput(long id, char *Class, descIO *Desc)
 {
 	static char buff[800];
@@ -222,7 +255,7 @@ bool ExecOutput(long id, char *Class, descIO *Desc)
 			cb += sprintf(buff+cb," %d\n", *(int*)Desc[i].ptr);
 			break;
 		case typNZLFLOAT:
-			if(*((double*)Desc[i].ptr) < 0.0001) {
+			if(fabs(*((double*)Desc[i].ptr)) < 1.0e-7) {
 				cb = last;
 				break;
 				}
@@ -351,7 +384,7 @@ bool ExecOutput(long id, char *Class, descIO *Desc)
 			break;
 		case typTEXT:
 			if(!*(char**)(Desc[i].ptr)) cb = last;
-			else cb += sprintf(buff+cb, "\"%s\"\n", *(char**)(Desc[i].ptr));
+			else cb += WriteEscString(*(char**)(Desc[i].ptr), buff, cb);
 			break;
 		case typTXTDEF:
 		case typPTRTXTDEF:
@@ -582,6 +615,7 @@ bool ExecInput(descIO *Desc)
 							*(GraphObj***)(Desc[j].ptr) = (GraphObj**)calloc(il, sizeof(GraphObj*));
 							}
 						if((gobs = *(GraphObj***)(Desc[j].ptr))){
+							i += 4;
 							while(tmp[i-1] != '{') i++; while(tmp[i-1] < 33) i++;
 							strcat(tmp, " ");
 							for( ;il >0; il--) {
@@ -641,7 +675,7 @@ bool ExecInput(descIO *Desc)
 						}
 					break;
 				case typTEXT:
-					for(i = strlen(tmp); i > 0 &&(tmp[i-1] < 33 || (tmp[i-1] == 
+					for(i = strlen(tmp); i > 0 &&(tmp[i-1] < 32 || (tmp[i-1] == 
 						'"' && tmp[i-2] != '\\') ||	(tmp[i-1] == '\\' && 
 						(mlines = true))); tmp[--i] = 0);
 					for(i = 0; tmp[i] && (tmp[i] < 33 || tmp[i] == '"'); i++);
@@ -695,6 +729,7 @@ bool SaveGraphAs(GraphObj *g)
 {
 	char *name = 0L;
 	int i;
+	bool bRet = true;
 
 	if(Notary || !g) {
 		ErrorBox("Output pending or\nno graph.");
@@ -723,8 +758,9 @@ bool SaveGraphAs(GraphObj *g)
 		else ErrorBox("Open failed for\noutput file.");
 		CloseOutputFile();
 		}
+	else bRet = false;
 	if(Notary) delete Notary;	Notary = 0L;
-	return true;
+	return bRet;
 }
 
 char *GraphToMem(GraphObj *g, long *size)
@@ -821,10 +857,10 @@ bool OpenGraph(GraphObj *root, char *name, unsigned char *mem)
 		hv = HashValue(tmp);
 		switch(hv) {
 		case 3895:		go = new Axis(FILE_READ);			break;
-		case 886:		go = new Bar(FILE_READ);			break;
+		case 7496002:	go = new Bar(FILE_READ);			break;
 		case 81384:		go = new Symbol(FILE_READ);			break;
 		case 62229:		go = new Bubble(FILE_READ);			break;
-		case 948:		go = new Box(FILE_READ);			break;
+		case 7892802:	go = new Box(FILE_READ);			break;
 		case 15411:		go = new Arrow(FILE_READ);			break;
 		case 1052406:	go = new ErrorBar(FILE_READ);		break;
 		case 324566:	go = new Whisker(FILE_READ);		break;
@@ -845,6 +881,7 @@ bool OpenGraph(GraphObj *root, char *name, unsigned char *mem)
 		case 386257:	go = new ellipse(FILE_READ);		break;
 		case 95680:		go = new mLabel(FILE_READ);			break;
 		case 4819316:	go = new PlotScatt(FILE_READ);		break;
+		case 117848:	go = new xyStat(FILE_READ);			break;
 		case 15935312:	go = new BubblePlot(FILE_READ);		break;
 		case 247376:	go = new BoxPlot(FILE_READ);		break;
 		case 317384:	go = new StackBar(FILE_READ);		break;
@@ -874,6 +911,7 @@ bool OpenGraph(GraphObj *root, char *name, unsigned char *mem)
 		case 68748:		go = new Legend(FILE_READ);			break;
 		case 66800:		go = new Grid3D(FILE_READ);			break;
 		case 967843:	go = new DefsRW(FILE_READ);			break;
+		case 66848:		go = new Func3D(FILE_READ);			break;
 		default:
 			sprintf(debug, "Object %ld in file\n(Class = \"%s\")\nhash #%d\nis unknown.",
 				id, tmp, hv);
@@ -891,6 +929,7 @@ bool OpenGraph(GraphObj *root, char *name, unsigned char *mem)
 	Cache->Close();
 	if((go = Notary->PopGO(lid))) {
 		go->Command(CMD_SET_DATAOBJ, 0L, 0L);
+		delete Notary;		Notary = 0L;
 		if(root->Id == GO_PAGE) {
 			if(go->Id == GO_PAGE){
 				if(!(root->parent->Command(CMD_DROP_GRAPH,(void *)go, 0L))) DeleteGO(go);
@@ -902,10 +941,8 @@ bool OpenGraph(GraphObj *root, char *name, unsigned char *mem)
 			}
 		if(go) go->Command(CMD_FILENAME, name, 0L);
 		}
-	delete Notary;
-	Notary = 0L;
-	delete Cache;
-	Cache = 0L;
+	if(Notary) delete Notary;		Notary = 0L;
+	delete Cache;					Cache = 0L;
 	return true;
 
 ReadErr:
@@ -1242,6 +1279,8 @@ Bar::FileIO(int rw)
 		BarFill.hatch = &HatchLine;
 		memcpy(&BarLine, defs.GetOutLine(), sizeof(LineDEF));
 		size = parent ? parent->GetSize(SIZE_BAR): defs.GetSize(SIZE_BAR);
+		mo = 0L;
+		mrc.left = mrc.right = mrc.top = mrc.bottom = 0;
 		return true;
 	case FILE_READ:
 		ExecInput(Desc);
@@ -1331,13 +1370,13 @@ DataLine::FileIO(int rw)
 		BgColor = defs.Color(COL_BG);
 		pts = 0L;		dirty = true;	mo = 0L;
 		mrc.left = mrc.right = mrc.top = mrc.bottom = 0;
-		min.fx = min.fy = max.fx = max.fy = 0.0f;
+		min.fx = min.fy = max.fx = max.fy = 0.0;
 		return true;
 	case FILE_READ:
 		ExecInput(Desc);
 		nPnt = i;
 		nPntSet = i-1;
-		if(file2)FileValues(file2, 1, 0.0f, 1.0f);
+		if(file2)FileValues(file2, 1, 0.0, 1.0);
 		else if(file1)FileValues(file1, 2, Start, fabs(Step) > defs.min4log ? Step : 1.0);
 		if(file1) free(file1);
 		if(file2) free(file2);
@@ -1776,6 +1815,7 @@ Line3D::FileIO(int rw)
 		{"ssRefX", typTEXT, &x_range, 0L},
 		{"ssRefY", typTEXT, &y_range, 0L},
 		{"ssRefZ", typTEXT, &z_range, 0L},
+		{"ssRef", typIPLST, &ssRef, &cssRef},
 		{"values", typLAST | typFPLST3D, &values, &nPts}};
 
 	switch(rw) {
@@ -1856,6 +1896,7 @@ mLabel::FileIO(int rw)
 		{"moveable", typNZINT, &moveable, 0L},
 		{"Pos", typNZLFPOINT, &fPos, 0L},
 		{"Dist", typNZLFPOINT, &fDist, 0L},
+		{"lspc", typLFLOAT, &lspc, 0L},
 		{"Flags", typDWORD, &flags, 0L},
 		{"TxtDef", typTXTDEF, &TextDef, 0L},
 		{"Lines", typLAST | typOBJLST, &Lines, &nLines}};
@@ -1878,7 +1919,7 @@ mLabel::FileIO(int rw)
 		TextDef.Style = TXS_NORMAL;
 		TextDef.Font = FONT_HELVETICA;
 		TextDef.text = 0L;
-		undo_flags = 0L;
+		undo_flags = 0L;	lspc = 1.0;
 		curr_z = 0.0;		is3D = false;
 		return true;
 	case FILE_READ:
@@ -2116,6 +2157,7 @@ bool
 PlotScatt::FileIO(int rw)
 {
 	descIO Desc[] = {
+		{"hide", typNZINT, &hidden, 0L},
 		{"Bounds", typFRECT, &Bounds, 0L},
 		{"DefSym", typNZINT, &DefSym, 0L},
 		{"baDist", typLFPOINT, &BarDist, 0L},
@@ -2163,6 +2205,51 @@ PlotScatt::FileIO(int rw)
 	return false;
 }
 
+bool
+xyStat::FileIO(int rw)
+{
+	descIO Desc[] = {
+		{"Type", typNZINT, &type, 0L},
+		{"hide", typNZINT, &hidden, 0L},
+		{"Bounds", typFRECT, &Bounds, 0L},
+		{"DefSym", typNZINT, &DefSym, 0L},
+		{"baDist", typLFPOINT, &BarDist, 0L},
+		{"confi", typLFLOAT, &ci, 0L},
+		{"xRange", typTEXT, &xRange, 0L},
+		{"yRange", typTEXT, &yRange, 0L},
+		{"prefix", typTEXT, &case_prefix, 0L},
+		{"x_axis", typNZINT, &use_xaxis, 0L},
+		{"y_axis", typNZINT, &use_yaxis, 0L},
+		{"Bars", typOBJLST, &Bars, &nPoints},
+		{"Symbols", typOBJLST, &Symbols, &nPoints},
+		{"PL", typGOBJ, &TheLine, 0L},
+		{"ErrBars", typOBJLST, &Errors, &nPoints},
+		{"Labels", typLAST | typOBJLST, &Labels, &nPoints}};
+	int i;
+
+	switch(rw) {
+	case INIT_VARS:
+		//most initialistion is done by PlotScatt::FileIO
+		curr_data = 0L;
+		case_prefix = 0L;
+		ci = 95.0;
+		return true;
+	case FILE_READ:
+		nPoints = 0L;
+		ExecInput(Desc);
+		ForEach(FE_PARENT, 0L, 0L);
+		return true;
+	case FILE_WRITE:
+		if(TheLine) TheLine->FileIO(rw);
+		if(Symbols) for(i = 0; i < nPoints; i++) if(Symbols[i]) Symbols[i]->FileIO(rw);
+		if(Errors) for(i = 0; i < nPoints; i++) if(Errors[i]) Errors[i]->FileIO(rw);
+		if(Labels) for(i = 0; i < nPoints; i++) if(Labels[i]) Labels[i]->FileIO(rw);
+		if(Bars) for(i = 0; i < nPoints; i++) if(Bars[i]) Bars[i]->FileIO(rw);
+		return ExecOutput(Notary->RegisterGO(this), "xyStat", Desc);
+		}
+	return false;
+}
+
 void
 FreqDist::RegGO(void *n)
 {
@@ -2179,6 +2266,7 @@ FreqDist::FileIO(int rw)
 {
 	descIO Desc[] = {
 		{"Type", typNZINT, &type, 0L},
+		{"hide", typNZINT, &hidden, 0L},
 		{"ssRef", typTEXT, &ssRef, 0L},
 		{"x_axis", typNZINT, &use_xaxis, 0L},
 		{"y_axis", typNZINT, &use_yaxis, 0L},
@@ -2234,6 +2322,7 @@ Regression::FileIO(int rw)
 {
 	descIO Desc[] = {
 		{"Type", typNZINT, &type, 0L},
+		{"hide", typNZINT, &hidden, 0L},
 		{"Bounds", typFRECT, &Bounds, 0L},
 		{"xRange", typTEXT, &xRange, 0L},
 		{"yRange", typTEXT, &yRange, 0L},
@@ -2280,6 +2369,7 @@ bool
 BubblePlot::FileIO(int rw)
 {
 	descIO Desc[] = {
+		{"hide", typNZINT, &hidden, 0L},
 		{"Bounds", typFRECT, &Bounds, 0L},
 		{"x_axis", typNZINT, &use_xaxis, 0L},
 		{"y_axis", typNZINT, &use_yaxis, 0L},
@@ -2330,6 +2420,7 @@ PolarPlot::FileIO(int rw)
 {
 	descIO Desc[] = {
 		{"Type", typNZINT, &type, 0L},
+		{"hide", typNZINT, &hidden, 0L},
 		{"Bounds", typFRECT, &Bounds, 0L},
 		{"ang_offs", typLFLOAT, &offs, 0L},
 		{"Plots", typOBJLST, &Plots, (long*)&nPlots},
@@ -2366,6 +2457,7 @@ BoxPlot::RegGO(void *n)
 		if(Boxes) for(i = 0; i < nPoints; i++) if(Boxes[i]) Boxes[i]->RegGO(n);
 		if(Whiskers) for(i = 0; i < nPoints; i++) if(Whiskers[i]) Whiskers[i]->RegGO(n);
 		if(Symbols) for(i = 0; i < nPoints; i++) if(Symbols[i]) Symbols[i]->RegGO(n);
+		if(Labels) for(i = 0; i < nPoints; i++) if(Labels[i]) Labels[i]->RegGO(n);
 		if(TheLine) TheLine->RegGO(n);
 		((notary*)n)->AddRegGO(this);
 		}
@@ -2375,13 +2467,21 @@ bool
 BoxPlot::FileIO(int rw)
 {
 	descIO Desc[] = {
+		{"Type", typNZINT, &type, 0L},
+		{"hide", typNZINT, &hidden, 0L},
 		{"Bounds", typFRECT, &Bounds, 0L},
+		{"xRange", typTEXT, &xRange, 0L},
+		{"yRange", typTEXT, &yRange, 0L},
+		{"prefix", typTEXT, &case_prefix, 0L},
 		{"boDist", typLFPOINT, &BoxDist, 0L},
+		{"ci_box", typNZLFLOAT, &ci_box, 0L},
+		{"ci_err", typNZLFLOAT, &ci_err, 0L},
 		{"x_axis", typNZINT, &use_xaxis, 0L},
 		{"y_axis", typNZINT, &use_yaxis, 0L},
 		{"Boxes", typOBJLST, &Boxes, &nPoints},
 		{"Whiskers", typOBJLST, &Whiskers, &nPoints},
 		{"Symbols", typOBJLST, &Symbols, &nPoints},
+		{"Labels", typOBJLST, &Labels, &nPoints},
 		{"Line", typLAST | typGOBJ, &TheLine, 0L}};
 	int i;
 
@@ -2393,6 +2493,7 @@ BoxPlot::FileIO(int rw)
 			sprintf(TmpTxt, "boxes (%s)", name);
 			free(name);		name=strdup(TmpTxt);
 			}
+		curr_data = 0L;
 		return true;
 	case FILE_READ:
 		return ExecInput(Desc);
@@ -2403,6 +2504,8 @@ BoxPlot::FileIO(int rw)
 			if(Whiskers[i]) Whiskers[i]->FileIO(rw);
 		if(Symbols) for(i = 0; i < nPoints; i++) 
 			if(Symbols[i]) Symbols[i]->FileIO(rw);
+		if(Labels) for(i = 0; i < nPoints; i++) 
+			if(Labels[i]) Labels[i]->FileIO(rw);
 		if(TheLine) TheLine->FileIO(rw);
 		return ExecOutput(Notary->RegisterGO(this), "BoxPlot", Desc);
 		}
@@ -2703,6 +2806,7 @@ Grid3D::RegGO(void *n)
 
 	if(n) {
 		if(lines) for(i = 0; i < nLines; i++) if(lines[i]) lines[i]->RegGO(n);
+		if(planes) for(i = 0; i < nPlanes; i++) if(planes[i]) planes[i]->RegGO(n);
 		((notary*)n)->AddRegGO(this);
 		}
 }
@@ -2715,7 +2819,9 @@ Grid3D::FileIO(int rw)
 		{"Start", typPOINT3D, &start, 0L},
 		{"Step", typPOINT3D, &step, 0L},
 		{"Line", typLINEDEF, &Line, 0L},
-		{"lines", typLAST | typOBJLST, &lines, &nLines}};
+		{"Fill", typFILLDEF, &Fill, 0L},
+		{"lines", typOBJLST, &lines, &nLines},
+		{"planes", typLAST | typOBJLST, &planes, &nPlanes}};
 	int i;
 
 	switch(rw) {
@@ -2734,9 +2840,11 @@ Grid3D::FileIO(int rw)
 		ExecInput(Desc);
 		//now set parent in all children
 		if(lines) for(i = 0; i < nLines; i++) if(lines[i]) lines[i]->parent = this;
+		if(planes) for(i = 0; i < nPlanes; i++) if(planes[i]) planes[i]->parent = this;
 		return true;
 	case FILE_WRITE:
 		if(lines) for(i = 0; i < nLines; i++) if(lines[i]) lines[i]->FileIO(rw);
+		if(planes) for(i = 0; i < nPlanes; i++) if(planes[i]) planes[i]->FileIO(rw);
 		return ExecOutput(Notary->RegisterGO(this), "Grid3D", Desc);
 		}
 	return false;
@@ -2775,16 +2883,14 @@ Function::RegGO(void *n)
 bool
 Function::FileIO(int rw)
 {
-	char *tmp_cmdxy = 0L, *tmp_param = 0L;
 	descIO Desc[] = {
 		{"x1", typNZLFLOAT, &x1, 0L},
 		{"x2", typNZLFLOAT, &x2, 0L},
 		{"xstep", typNZLFLOAT, &xstep, 0L},
 		{"Line", typLINEDEF, &Line, 0L},
-		{"f_xy", typTEXT, &tmp_cmdxy, 0L},
-		{"param", typTEXT, &tmp_param, 0L},
+		{"f_xy", typTEXT, &cmdxy, 0L},
+		{"param", typTEXT, &param, 0L},
 		{"DataLine", typLAST | typGOBJ, &dl, 0L}};
-	int i, j;
 
 	switch(rw) {
 	case SAVE_VARS:
@@ -2800,31 +2906,11 @@ Function::FileIO(int rw)
 		return true;
 	case FILE_READ:
 		ExecInput(Desc);
-		if(tmp_cmdxy && tmp_cmdxy[0]) cmdxy = tmp_cmdxy;
-		if(tmp_param && tmp_param[0]) param = tmp_param;
 		if(dl) dl->parent = this;
 		return true;
 	case FILE_WRITE:
 		if(dl) dl->FileIO(rw);
-		if(cmdxy) {
-			for(i = j = 0; cmdxy[i]; i++) {
-				if(cmdxy[i] == '\n') j += sprintf(TmpTxt+j, "\\n");
-				else TmpTxt[j++] = cmdxy[i];
-				}
-			TmpTxt[j] = 0;
-			if(i) tmp_cmdxy = strdup(TmpTxt);
-			}
-		if(param) {
-			for(i = j = 0; param[i]; i++) {
-				if(param[i] == '\n') j += sprintf(TmpTxt+j, "\\n");
-				else TmpTxt[j++] = param[i];
-				}
-			TmpTxt[j] = 0;
-			if(i) tmp_param = strdup(TmpTxt);
-			}
 		ExecOutput(Notary->RegisterGO(this), "Function", Desc);
-		if(tmp_cmdxy) free(tmp_cmdxy);
-		if(tmp_param) free(tmp_param);
 		}
 	return false;
 }
@@ -2843,7 +2929,6 @@ FitFunc::RegGO(void *n)
 bool
 FitFunc::FileIO(int rw)
 {
-	char *tmp_cmdxy = 0L, *tmp_parxy = 0L;
 	descIO Desc[] = {
 		{"ssXref", typTEXT, &ssXref, 0L},
 		{"ssYref", typTEXT, &ssYref, 0L},
@@ -2854,10 +2939,10 @@ FitFunc::FileIO(int rw)
 		{"chi2", typNZLFLOAT, &chi2, 0L},
 		{"maxiter", typNZINT, &maxiter, 0L},
 		{"Line", typLINEDEF, &Line, 0L},
-		{"f_xy", typTEXT, &tmp_cmdxy, 0L},
-		{"p_xy", typTEXT, &tmp_parxy, 0L},
+		{"f_xy", typTEXT, &cmdxy, 0L},
+		{"p_xy", typTEXT, &parxy, 0L},
 		{"Symbols", typLAST | typOBJLST, &Symbols, &nPoints}};
-	int i, j;
+	int i;
 
 	switch(rw) {
 	case SAVE_VARS:
@@ -2875,28 +2960,10 @@ FitFunc::FileIO(int rw)
 		return true;
 	case FILE_READ:
 		ExecInput(Desc);
-		if(tmp_cmdxy && tmp_cmdxy[0]) cmdxy = tmp_cmdxy;
-		if(tmp_parxy && tmp_parxy[0]) parxy = tmp_parxy;
 		if(Symbols) for(i = 0; i < nPoints; i++)
 			if(Symbols[i]) Symbols[i]->parent = this;
 		return true;
 	case FILE_WRITE:
-		if(cmdxy) {
-			for(i = j = 0; cmdxy[i]; i++) {
-				if(cmdxy[i] == '\n') j += sprintf(TmpTxt+j, "\\n");
-				else TmpTxt[j++] = cmdxy[i];
-				}
-			TmpTxt[j] = 0;
-			if(i) tmp_cmdxy = strdup(TmpTxt);
-			}
-		if(parxy) {
-			for(i = j = 0; parxy[i]; i++) {
-				if(parxy[i] == '\n') j += sprintf(TmpTxt+j, "\\n");
-				else TmpTxt[j++] = parxy[i];
-				}
-			TmpTxt[j] = 0;
-			if(i) tmp_parxy = strdup(TmpTxt);
-			}
 		if(Symbols) for(i = 0; i < nPoints; i++) if(Symbols[i]) Symbols[i]->FileIO(rw);
 		return ExecOutput(Notary->RegisterGO(this), "FitFunc", Desc);
 		}
@@ -3105,7 +3172,7 @@ Axis::FileIO(int rw)
 			}
 		if(axis) axis->owner = this;
 		if(axisLabel)axisLabel->parent = this;
-		if(Ticks && NumTicks) for(i = 0; i < NumTicks; i++) if(Ticks[i]) Ticks[i]->parent = this;
+		if(Ticks) for(i = 0; i < NumTicks; i++) if(Ticks[i]) Ticks[i]->parent = this;
 		return true;
 	case FILE_WRITE:
 		//do all ticks
@@ -3188,6 +3255,78 @@ Plot3D::FileIO(int rw)
 	return false;
 }
 
+void
+Func3D::RegGO(void *n)
+{
+}
+
+bool
+Func3D::FileIO(int rw)
+{
+	fPOINT3D rot_vec, rot_ang;
+	//DEBUG: must initialize rot_vec, rot_ang in case they are not present in file
+	descIO Desc[] = {
+		{"Type", typNZINT, &type, 0L},
+		{"xBounds", typLFPOINT, &xBounds, 0L},
+		{"yBounds", typLFPOINT, &yBounds, 0L},
+		{"zBounds", typLFPOINT, &zBounds, 0L},
+		{"Corner1", typPOINT3D, &cub1, 0L},
+		{"Corner2", typPOINT3D, &cub2, 0L},
+		{"Center", typPOINT3D, &rotC, 0L},
+		{"rot_vec", typPOINT3D, &rot_vec, 0L},
+		{"rot_ang", typPOINT3D, &rot_ang, 0L},
+		{"Axes", typOBJLST, &Axes, (long*)&nAxes},
+		{"Plots", typOBJLST, &plots, (long*)&nPlots},
+		{"x1", typNZLFLOAT, &x1, 0L},
+		{"x2", typNZLFLOAT, &x2, 0L},
+		{"xstep", typNZLFLOAT, &xstep, 0L},
+		{"z1", typNZLFLOAT, &x1, 0L},
+		{"z2", typNZLFLOAT, &x2, 0L},
+		{"zstep", typNZLFLOAT, &xstep, 0L},
+		{"g_idx", typNZINT, &g_idx, 0L},
+		{"Line", typLINEDEF, &Line, 0L},
+		{"Fill", typFILLDEF, &Fill, 0L},
+		{"f_xz", typTEXT, &cmdxy, 0L},
+		{"param", typLAST | typTEXT, &param, 0L}};
+	int i;
+
+	switch(rw) {
+	case SAVE_VARS:
+		return SaveVarGO(Desc);
+	case INIT_VARS:
+		x1 = -20.0;		x2 = 20.0;	xstep = 2.0;
+		z1 = -20.0;		z2 = 20.0;	zstep = 2.0;
+		gda = 0L;		gob = 0L;
+		g_idx = 0;		param = cmdxy = 0L;
+		Line.width = defs.GetSize(SIZE_HAIRLINE);
+		Line.patlength = defs.GetSize(SIZE_PATLENGTH);
+		Line.color = Line.pattern = 0x0L;
+		Fill.color = 0x00c0c0c0;
+		Fill.color2 = 0x00ffffff;
+		Fill.hatch = 0L;
+		Fill.type = FILL_LIGHT3D;
+		if(name) {
+			sprintf(TmpTxt, "3D function (Plot %d)", cPlots);
+			free(name);		name=strdup(TmpTxt);
+			}
+		return true;
+	case FILE_READ:
+		ExecInput(Desc);
+		RotDef[0] = rot_vec.fx;	RotDef[1] = rot_vec.fy;	RotDef[2] = rot_vec.fz;
+		RotDef[3] = rot_ang.fx;	RotDef[4] = rot_ang.fy;	RotDef[5] = rot_ang.fz;
+		return true;
+	case FILE_WRITE:
+		rot_vec.fx = RotDef[0];	rot_vec.fy = RotDef[1];	rot_vec.fz = RotDef[2];
+		rot_ang.fx = RotDef[3];	rot_ang.fy = RotDef[4];	rot_ang.fz = RotDef[5];
+		//do all plots
+		for(i = 0; plots && i< nPlots; i++) if(plots[i]) plots[i]->FileIO(rw);
+		//do all axes
+		if(Axes) for(i = 0; i< nAxes; i++) if(Axes[i]) Axes[i]->FileIO(rw);
+		ExecOutput(Notary->RegisterGO(this), "Func3D", Desc);
+		}
+	return false;
+}
+
 void
 Graph::RegGO(void *n)
 {
@@ -3265,7 +3404,7 @@ Graph::FileIO(int rw)
 			if(Axes[i] && Axes[i]->GetAxis() == &x_axis) ixax = i;
 			else if(Axes[i] && Axes[i]->GetAxis() == &y_axis) iyax = i;
 			}
-		if(Id == GO_GRAPH && parent && parent->Id != GO_PAGE)RegGO(Notary);
+		if(Id == GO_GRAPH)RegGO(Notary);
 		//do all plots
 		if(Plots) for(i = 0; i< NumPlots; i++) if(Plots[i]) Plots[i]->FileIO(rw);
 		//do all axes
@@ -3281,7 +3420,8 @@ Page::RegGO(void *n)
 	int i;
 
 	if(n) {
-		if(Plots) for(i = 0; i< NumPlots; i++) if(Plots[i]) Plots[i]->RegGO(n);
+		if(Plots) for(i = 0; i< NumPlots; i++) 
+			if(Plots[i] && Plots[i]->Id != GO_GRAPH) Plots[i]->RegGO(n);
 		((notary*)n)->AddRegGO(this);
 		}
 }
@@ -3315,6 +3455,7 @@ Page::FileIO(int rw)
 	case FILE_WRITE:
 		//do all plots
 		bModified = false;
+		if(Id == GO_PAGE)RegGO(Notary);
 		if(Plots) for(i = 0; i< NumPlots; i++) if(Plots[i]) Plots[i]->FileIO(rw);
 		return ExecOutput(Notary->RegisterGO(this), "Page", Desc);
 		}
@@ -3328,6 +3469,7 @@ DefsRW::FileIO(int rw)
 		{"dUnits", typINT, &defs.dUnits, 0L},
 		{"cUnits", typINT, &defs.dUnits, 0L},
 		{"dtHeight", typINT, &dlgtxtheight, 0L},
+		{"ss_txt", typLFLOAT, &defs.ss_txt, 0L},
 		{"File1", typTEXT, &defs.File1, 0L},
 		{"File2", typTEXT, &defs.File2, 0L},
 		{"File3", typTEXT, &defs.File3, 0L},
