@@ -117,14 +117,15 @@ void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 {
 	LineDEF Line = {.1f, 1.0f, 0x0L, 0x0L};
 	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
-	POINT pts[10];
-	int ix, iy, np;
+	POINT *pts;
+	int i, ix, iy, np;
 
+	if(!(pts=(POINT*)malloc(sizeof(POINT)*(rec->right-rec->left)))) return;
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
 		o->SetLine(&Line);
@@ -189,17 +190,49 @@ void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			pts[np].x = pts[np-1].x;		pts[np++].y = rec->top+15;
 			if(id == 209) pts[np-1].y -= 7;
 			break;
+		case 210:
+			pts[0].x = rec->left +9;	pts[0].y = iy+4;	pts[1].x = pts[0].x+1;
+			for(i = 0; i < (rec->right - rec->left - 18); i++) {
+				pts[1].y = 4 + iy + iround(pow(20.0, 1.0+((double)-i)/30.0) * -sin(((double)i)/4.0));
+				o->oSolidLine(pts);
+				pts[0].x++;		pts[1].x++;		pts[0].y = pts[1].y;	
+				}
+			o->oCircle(rec->left+7, iy+4, rec->left+12, iy +9);
+			o->oCircle(rec->left+12, iy-10, rec->left+17, iy -5);
+			o->oCircle(rec->right-19, iy+5, rec->right-24, iy +10);
+			o->oCircle(rec->right-9, iy, rec->right-14, iy+5);
+			break;
+		case 211:
+			pts[0].y = rec->top +9;	pts[0].x = ix;	pts[1].y = pts[0].y+1;
+			for(i = 0; i < (rec->bottom - rec->top - 18); i++) {
+				pts[1].x = ix + iround(pow(20.0, 1.0+((double)-i)/50.0) * -sin(((double)i)/4.0));
+				o->oSolidLine(pts);
+				pts[0].y++;		pts[1].y++;		pts[0].x = pts[1].x;	
+				}
+			o->oCircle(ix-3, rec->top + 7, ix+2, rec->top + 12);
+			o->oCircle(rec->left+11, iy-10, rec->left+16, iy -5);
+			o->oCircle(ix+3, rec->top + 27, ix+8, rec->top + 32);
+			o->oCircle(ix-5, iy+12, ix, iy + 17);
+			break;
 			}
 		if(np) o->oPolyline(pts, np);
-		o->oCircle(ix-2, iy-2, ix+2, iy+2);
+		switch(id) {
+		case 201:	case 202:	case 203:	case 204:	case 205:
+		case 206:	case 207:	case 208:	case 209:
+			o->oCircle(ix-2, iy-2, ix+2, iy+2);
 #ifdef _WINDOWS
-		o->oCircle(rec->left+13, rec->bottom-13, rec->left+17, rec->bottom-17);
-		o->oCircle(rec->right-13, rec->top+13, rec->right-17, rec->top+17);
+			o->oCircle(rec->left+13, rec->bottom-13, rec->left+17, rec->bottom-17);
+			o->oCircle(rec->right-13, rec->top+13, rec->right-17, rec->top+17);
 #else
-		o->oCircle(rec->left+13, rec->bottom-14, rec->left+17, rec->bottom-16);
-		o->oCircle(rec->right-14, rec->top+13, rec->right-16, rec->top+17);
+			o->oCircle(rec->left+13, rec->bottom-14, rec->left+17, rec->bottom-16);
+			o->oCircle(rec->right-14, rec->top+13, rec->right-16, rec->top+17);
 #endif
+			break;
+		case 210:	case 211:
+			break;
+			}
 		o->UpdateRect(rec, false);
+		free(pts);
 		break;
 		}
 }
@@ -219,8 +252,8 @@ void OD_ErrBarTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
 		o->SetLine(&Line);
@@ -283,8 +316,8 @@ void OD_WhiskerTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
 		o->SetLine(&Line);
@@ -344,8 +377,8 @@ void OD_PolarTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
 		o->SetLine(&Line);
@@ -388,7 +421,7 @@ void OD_PolarTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			o->oPolygon(pts, 5);
 			break;
 		case 204:
-			if(cmd == OD_DRAWNORMAL) FillG.color = 0x00d0d0d0L;
+			if(cmd == OD_DRAWNORMAL) FillG.color = 0x00e8e8e8L;
 			o->SetFill(&FillG);
 			o->oCircle(ix-6, rec->top+5, ix+6, iy+6);
 			memcpy(&td, &o->TxtSet, sizeof(TextDEF));
@@ -429,8 +462,8 @@ void OD_PieTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)/2;
 		iy = (rec->top +rec->bottom)/2;
 		o->SetLine(&Line);
@@ -525,8 +558,8 @@ void OD_BreakTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		ix = (rec->left + rec->right)>>1;
 		iy = (rec->top +rec->bottom)>>1;
 		o->SetLine(&Line);
@@ -617,11 +650,12 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		iy = (rec->top +rec->bottom)>>1;
 		switch(id) {
 		case 560:	case 561:	case 562:	case 563:	case 564:		//3D axes
+		case 565:
 			OD_AxisTempl3D(cmd, par, rec, o, data, 410+AxisTempl3D);
 			break;
 		default:
-			Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-			Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+			Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+			Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 			o->SetLine(&Line);
 			pts[0].x = pts[3].x = pts[4].x = rec->left;
 			pts[0].y = pts[1].y = pts[4].y = rec->top;
@@ -844,6 +878,41 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			pts[0].y += 9;	pts[1].y += 10;	pts[2].y = pts[1].y;
 			o->oPolyline(pts, 3, 0L);
 			break;
+		case 532:
+			pts[0].x = rec->left +13;	pts[0].y = rec->top+8;	
+			pts[1].x = rec->left +13;	pts[1].y = iy;
+			o->oSolidLine(pts);
+			pts[0].x -= 3;		pts[1].x += 3;
+			pts[0].y = pts[1].y = rec->top+8;
+			o->oSolidLine(pts);
+			pts[0].y = pts[1].y = iy;
+			o->oSolidLine(pts);
+			pts[0].x = ix;	pts[0].y = iy-8;	
+			pts[1].x = ix;	pts[1].y = rec->bottom-13;
+			o->oSolidLine(pts);
+			pts[0].x -= 3;		pts[1].x += 3;
+			pts[0].y = pts[1].y = iy-8;
+			o->oSolidLine(pts);
+			pts[0].y = pts[1].y = rec->bottom-13;
+			o->oSolidLine(pts);
+			pts[0].x = rec->right -13;	pts[0].y = rec->top+10;	
+			pts[1].x = rec->right -13;	pts[1].y = iy-6;
+			o->oSolidLine(pts);
+			pts[0].x -= 3;		pts[1].x += 3;
+			pts[0].y = pts[1].y = rec->top+10;
+			o->oSolidLine(pts);
+			pts[0].y = pts[1].y = iy-6;
+			o->oSolidLine(pts);
+			pts[0].x = rec->left+13;	pts[1].x = ix;		pts[2].x = rec->right-13;
+			pts[0].y = (rec->top+8+iy)>>1;
+			pts[1].y = (rec->bottom-13 + iy -8)>>1;
+			pts[2].y = (rec->top+10+iy-6)>>1;
+			o->oPolyline(pts, 3, 0L);
+			o->SetFill(&FillY);
+			o->oCircle(pts[0].x-3, pts[0].y-3, pts[0].x+3, pts[0].y+3);
+			o->oCircle(pts[1].x-3, pts[1].y-3, pts[1].x+3, pts[1].y+3);
+			o->oCircle(pts[2].x-3, pts[2].y-3, pts[2].x+3, pts[2].y+3);
+			break;
 		case 540:
 			o->SetFill(&FillR);
 			o->oRectangle(rec->left+8, rec->bottom-8, rec->left+16, rec->bottom-3);
@@ -1012,6 +1081,32 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			pts[4].x = pts[0].x;		pts[4].y = pts[0].y;
 			o->oPolygon(pts, 5, 0L);
 			break;
+		case 565:
+			o->SetLine(&rLine);
+			pts[0].x = ix-16;			pts[0].y = iy-2;
+			pts[1].x = ix+4;			pts[1].y = iy+6;
+			for(i = 0; i < 4; i++) {
+				o->oSolidLine(pts);
+				pts[0].x += 4;			pts[1].x += 4;
+				pts[0].y -= 4;			pts[1].y -= 4;
+				}
+			pts[0].x = ix+4;			pts[0].y = iy+6;
+			pts[1].x -= 2;				pts[1].y += 4;
+			for(i = 0; i < 5; i++) {
+				o->oSolidLine(pts);
+				pts[0].x -= 5;			pts[1].x -= 5;
+				pts[0].y -= 2;			pts[1].y -= 2;
+				}
+			memcpy(&td, &o->TxtSet, sizeof(TextDEF));
+			memcpy(&otd, &o->TxtSet, sizeof(TextDEF));
+			td.Align = TXA_HCENTER | TXA_VTOP;
+			td.Style = TXS_NORMAL;
+			td.Mode = TXM_TRANSPARENT;
+			td.ColTxt = 0x00c00000L;
+			o->SetTextSpec(&td);
+			o->oTextOut(ix, iy+4, "f(x,z)", 0);
+			o->SetTextSpec(&otd);
+			break;
 			}
 		o->UpdateRect(rec, false);
 		break;
@@ -1033,8 +1128,8 @@ void OD_AxisTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		o->SetLine(&Line);
 		pts[0].x = pts[3].x = pts[4].x = rec->left;
 		pts[0].y = pts[1].y = pts[4].y = rec->top;
@@ -1152,8 +1247,8 @@ void OD_AxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		o->SetLine(&Line);
 		pts[0].x = pts[3].x = pts[4].x = rec->left;
 		pts[0].y = pts[1].y = pts[4].y = rec->top;
@@ -1161,9 +1256,10 @@ void OD_AxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
 		pts[2].y = pts[3].y = rec->bottom-1;
 		o->oPolyline(pts, 5);
 		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
+		o->SetLine(&Line);						o->SetFill(&Fill);
 		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00c0c0c0L : 0x0L;
+		o->SetLine(&Line);						o->SetFill(&Fill);
 		switch(id) {
 		case 410:	case 411:
 			pts[0].x = rec->left+20;			pts[0].y = rec->bottom-14;
@@ -1199,6 +1295,7 @@ void OD_AxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
 			o->oSolidLine(pts);
 			}
 		o->UpdateRect(rec, false);
+		Line.color = 0x00000000L;		o->SetLine(&Line);
 		break;
 		}
 }
@@ -1217,8 +1314,8 @@ void OD_NewAxisTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 	switch(cmd) {
 	case OD_DRAWNORMAL:
 	case OD_DRAWSELECTED:
-		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00d0d0d0L;
-		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00d0d0d0L;
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
 		o->SetLine(&Line);
 		ix = (rec->right + rec->left)>>1;
 		iy = (rec->bottom + rec->top)>>1;
