@@ -48,7 +48,7 @@ QWidget *MainWidget =0L;
 POINT CurrWidgetPos = {0,0};
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Menu tem identifiers: synchronize this table with rlplot.rc for
+// Menu item identifiers: synchronize this table with rlplot.rc for
 //    compatibility with windows
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #define CM_OPEN        500
@@ -71,37 +71,39 @@ POINT CurrWidgetPos = {0,0};
 #define CM_PRINT       517
 #define CM_EXPORT      518
 #define CM_DELOBJ      519
-#define CM_REBOOT      520
-#define CM_SHUTDOWN    521
-#define CM_DEFAULTS    522
-#define CM_COPY        523
-#define CM_PASTE       524
-#define CM_UPDATE      525
-#define CM_ADDAXIS     526
-#define CM_UNDO        527
-#define CM_ZOOMIN      528
-#define CM_ZOOMOUT     529
-#define CM_ZOOMFIT     530
-#define CM_FILE1       531
-#define CM_FILE2       532
-#define CM_FILE3       533
-#define CM_FILE4       534
-#define CM_FILE5       535
-#define CM_FILE6       536
-#define CM_FILLRANGE   537
-#define CM_CUT         538
-#define CM_LEGEND      539
-#define CM_LAYERS      540
+#define CM_DEFAULTS    520
+#define CM_COPY        521
+#define CM_PASTE       522
+#define CM_UPDATE      523
+#define CM_ADDAXIS     524
+#define CM_UNDO        525
+#define CM_ZOOMIN      526
+#define CM_ZOOMOUT     527
+#define CM_ZOOMFIT     528
+#define CM_FILE1       529
+#define CM_FILE2       530
+#define CM_FILE3       531
+#define CM_FILE4       532
+#define CM_FILE5       533
+#define CM_FILE6       534
+#define CM_FILLRANGE   535
+#define CM_CUT         536
+#define CM_LEGEND      537
+#define CM_LAYERS      538
+#define CM_INSROW      539
+#define CM_INSCOL      540
+#define CM_DELROW      541
+#define CM_DELCOL      542
 
-#define CM_T_STANDARD  550
-#define CM_T_DRAW      551
-#define CM_T_POLYLINE  552
-#define CM_T_POLYGON   553
-#define CM_T_RECTANGLE 554
-#define CM_T_ROUNDREC  555
-#define CM_T_ELLIPSE   556
-#define CM_T_ARROW     557
-#define CM_T_TEXT      558
+#define CM_T_STANDARD  560
+#define CM_T_DRAW      561
+#define CM_T_POLYLINE  562
+#define CM_T_POLYGON   563
+#define CM_T_RECTANGLE 564
+#define CM_T_ROUNDREC  565
+#define CM_T_ELLIPSE   566
+#define CM_T_ARROW     567
+#define CM_T_TEXT      568
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Exute a file open dialog for the different situations
@@ -219,6 +221,20 @@ bool YesNoBox(char *Msg)
 	return true;
 }
 
+int YesNoCancelBox(char *Msg)
+{
+	int res;
+
+	res = QMessageBox::information(QAppl->focusWidget(), "RLPlot", Msg,
+		"&Yes", "&No", "&Cancel", 0, -1);
+	switch(res) {
+	case 0:		return 1;
+	case 1:		return 0;
+	default:	return 2;
+		}
+	return 0;
+}
+
 void Qt_Box()
 {
 	QMessageBox::aboutQt(QAppl->focusWidget(), "RLPlot uses Qt");
@@ -291,8 +307,8 @@ void ShowCopyMark(anyOutput *out, RECT *mrk, int nRec)
 		UpdateMinMaxRect(&rCopyMark, mrk[i].left, mrk[i].top);
 		UpdateMinMaxRect(&rCopyMark, mrk[i].right, mrk[i].bottom);
 		}
-	cTxtCur->bmCopyMark = new BitMapQT(rCopyMark.right - rCopyMark.left, 
-		rCopyMark.bottom - rCopyMark.top, out->hres, out->vres);
+	cTxtCur->bmCopyMark = new BitMapQT(rCopyMark.right - rCopyMark.left+1, 
+		rCopyMark.bottom - rCopyMark.top+1, out->hres, out->vres);
 }
 
 LineDEF liCopyMark1 = {0.0f, 1.0f, 0x00ffffffL, 0L};
@@ -444,7 +460,7 @@ KSpreadTextDrag::format(int i) const
 const char *
 KSpreadTextDrag::selectionMimeType()
 {
-	return(MIME_KSPREAD);
+	return(MIME_RLPLOT);
 }
 
 RLPGraphDrag::RLPGraphDrag(QWidget *dragSource, const char *name)
@@ -523,10 +539,10 @@ void TestClipboard(GraphObj *g)
 {
 
 	QClipboard *cb;
-	QMimeSource *mime;
+	QDragObject *mime;
 
 	if(!(cb = QAppl->clipboard()))return;
-	if(!(mime = cb->data()))return;
+	if(!(mime = (QDragObject *)cb->data()))return;
 	if(g->Id == GO_SPREADDATA) {
 		if(mime->provides(MIME_RLPLOT)){
 			QByteArray b = mime->encodedData(MIME_RLPLOT);
@@ -596,6 +612,47 @@ void EmptyClip()
 	QAppl->clipboard()->clear();
 }
 
+void CopyText(char *txt, int len)
+{
+	QClipboard *cb;
+	char tmp_txt[90];
+
+	if(!(cb = QAppl->clipboard()) || !txt || !txt[0])return;
+	EmptyClip();
+	if(!len) len = strlen(txt);
+	memcpy(tmp_txt, txt, len);		tmp_txt[len] = 0;
+#ifdef Q_CHECK_PTR				//n.a. in Qt version 2
+	cb->setText(tmp_txt, QClipboard::Clipboard);
+	cb->setText(tmp_txt, QClipboard::Selection);
+#else
+	cb->setText(tmp_txt);
+#endif
+}
+
+unsigned char* PasteText()
+{
+	QClipboard *cb;
+	QMimeSource *mime;
+	unsigned char *tmp_txt = 0L;
+
+	if(!(cb = QAppl->clipboard())) return 0L;
+#ifdef Q_CHECK_PTR				//n.a. in Qt version 2
+	QString qstr = cb->text(QClipboard::Clipboard);
+#else
+	QString qstr = cb->text();
+#endif
+	if(qstr.length()) tmp_txt = (unsigned char*) strdup(qstr);
+	else {
+#ifdef Q_CHECK_PTR				//n.a. in Qt version 2
+		qstr = cb->text(QClipboard::Selection);
+#else
+		QString qstr = cb->text();
+#endif
+		if(qstr.length()) tmp_txt = (unsigned char*) strdup(qstr);
+		}
+	return tmp_txt;
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Get display (desktop) size
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -641,10 +698,10 @@ bool com_TextOut(int x, int y, char *ctxt, TextDEF *TxtSet, QPainter *qP, anyOut
 	oldBrush = qP->brush();		dxf = qP->worldMatrix();
 	oldPen = currPen = qP->pen();
 	o->oGetTextExtent(ctxt, -1, &w, &h);
-	if(TxtSet->Align & TXA_VCENTER) iy = y + h/3;
+	if(TxtSet->Align & TXA_VCENTER) iy = y + iround(TxtSet->iSize *0.4);
 	else if(TxtSet->Align & TXA_VBOTTOM) iy = y;
-	else iy = y + iround(h*.8);
-	if(TxtSet->Align & TXA_HCENTER) ix = x - w/2;
+	else iy = y + TxtSet->iSize + 1;
+	if(TxtSet->Align & TXA_HCENTER) ix = x - (w >> 1);
 	else if(TxtSet->Align & TXA_HRIGHT) ix = x - w;
 	else ix = x;
 	currPen.setColor(SwapRB(TxtSet->ColTxt));
@@ -661,16 +718,8 @@ bool com_TextOut(int x, int y, char *ctxt, TextDEF *TxtSet, QPainter *qP, anyOut
 			currPen.setColor(SwapRB(TxtSet->ColTxt));
 			qP->setPen(currPen);
 			}
-		if(TxtSet->Style & TXS_SUB) {
-			if((TxtSet->Align & TXA_VCENTER) == TXA_VCENTER) iy += o->un2iy(TxtSet->fSize*0.4);
-			else if((TxtSet->Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy += o->un2iy(TxtSet->fSize*0.2);
-			else if((TxtSet->Align & TXA_VTOP) == TXA_VTOP) iy += o->un2iy(TxtSet->fSize*.6);
-			}
-		else if(TxtSet->Style & TXS_SUPER) {
-			if((TxtSet->Align & TXA_VCENTER) == TXA_VCENTER) iy -= o->un2iy(TxtSet->fSize*0.4);
-			else if((TxtSet->Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy -= o->un2iy(TxtSet->fSize*0.6);
-			else if((TxtSet->Align & TXA_VTOP) == TXA_VTOP) iy += o->un2iy(TxtSet->fSize*.0);
-			}
+		if(TxtSet->Style & TXS_SUB) iy += o->un2iy(TxtSet->fSize*0.4);
+		else if(TxtSet->Style & TXS_SUPER) iy -= o->un2iy(TxtSet->fSize*0.4);
 		qP->drawText(ix-x, iy-y, txt, -1);
 		}
 	else {
@@ -682,16 +731,8 @@ bool com_TextOut(int x, int y, char *ctxt, TextDEF *TxtSet, QPainter *qP, anyOut
 			currPen.setColor(SwapRB(TxtSet->ColTxt));
 			qP->setPen(currPen);
 			}
-		if(TxtSet->Style & TXS_SUB) {
-			if((TxtSet->Align & TXA_VCENTER) == TXA_VCENTER) iy += o->un2iy(TxtSet->fSize*0.4);
-			else if((TxtSet->Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy += o->un2iy(TxtSet->fSize*0.2);
-			else if((TxtSet->Align & TXA_VTOP) == TXA_VTOP) iy += o->un2iy(TxtSet->fSize*.6);
-			}
-		else if(TxtSet->Style & TXS_SUPER) {
-			if((TxtSet->Align & TXA_VCENTER) == TXA_VCENTER) iy -= o->un2iy(TxtSet->fSize*0.4);
-			else if((TxtSet->Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy -= o->un2iy(TxtSet->fSize*0.6);
-			else if((TxtSet->Align & TXA_VTOP) == TXA_VTOP) iy += o->un2iy(TxtSet->fSize*.0);
-			}
+		if(TxtSet->Style & TXS_SUB) iy += o->un2iy(TxtSet->fSize*0.4);
+		else if(TxtSet->Style & TXS_SUPER) iy -= o->un2iy(TxtSet->fSize*0.4);
 		qP->drawText(ix, iy, txt, -1);
 		}
 	qP->setPen(oldPen);
@@ -815,52 +856,213 @@ static char *critical_xpm[]={
 "...........aaaaaaaaaaa..........",
 ".............aaaaaaa............"};
 
+//thanks to Markus Bongard for the following icon
 static char *RLPlot_xpm[]={
-"32 32 12 1",
-"j c #000083",
-"i c #0000ff",
-"d c #008100",
-"e c #00ff00",
-"b c #00ffff",
-"g c #830000",
-"c c #838100",
-"a c #838183",
-". c #c5c2c5",
-"h c #ff0000",
-"f c #ffff00",
-"# c #ffffff",
-".##a###a##bbbbbbbbbbbbbbbbbbbbbb",
-"##a###a###bbbbbbbbbbbbbbbbbbbbbb",
-"#a###a#.#a.b.b.b.b.b.b.b.b.b.b.b",
-"a#.#a#.#a#bbbbbbbbbbbbbbbbbbbbbb",
-"###a##.cccbbbbbbbbbbbbbdddbbbbbb",
-".#a###a#c#bbbbbbbbbbbbbbdbbbbbbb",
-"#a#.#a#.c.b.b.b.b.b.b.b.d.b.b.b.",
-"a###a###c#bbbbbbbbbbbdddddddbbbb",
-"#.#a#.#ac#bbbbbbbbbbbdeedeedbbbb",
-".#a.#cccccccbbbbbbbbbdeedeedbbbb",
-"#a###cffcffc.b.ggg.b.deddded.b.b",
-"a###acffcffcbbbbgbbbbdeeeeedbbbb",
-"..#a.cffcffcbbbbgbbbbdeeeeedbbbb",
-"##a##cffcffcbgggggggbdeeeeedbbbb",
-"#a#.#cfcccfcbghhghhgbdeeeeedb.b.",
-"a###acfffffcbghhghhgbdeeeeedbbbb",
-"#.#a#cfffffcbghggghgbdeeeeedbbbb",
-".#a.#cfffffcbghhhhhgbdeeeeedbbbb",
-"#a###cfffffcbghhhhhg.deeeeed.b.b",
-"a#.#acfffffcbghhhhhgbdeeeeedbbbb",
-"###a#cfffffcbghhhhhgbdeeeeedbbbb",
-".#a#.cfffffcighhhhhgideeeeediiii",
-"#a.##cfffffcighhhhhgideeeeedjiij",
-"a###acccccccigggggggidddddddiiii",
-"#.#a###ijiiijiiijiiijiiijiiiijii",
-".#a#.#ijijjiijiiijiiijiiijiiiiii",
-"#a###ijjiiijiijiijjijiijiijiijii",
-"a##.iijiijijjiijjiiijjijjiijjiij",
-".##jijiijjjiijjjiijjjiijjjiijjji",
-"##jjijijiijjjjijjjjijjjjijjjjijj",
-".jjjjjijjjjjijjjjijjjijjjjjjjjij",
-"jjijjjjjjjjjjjjijjjjjjjjjjijjjjj"};
+/* width height ncolors chars_per_pixel */
+"32 32 169 2",
+/* colors */
+"AA c #FFFFFFFFFFFF",
+"BA c #FFFFF7F79494",
+"CA c #FFFFF7F78484",
+"DA c #FFFFF7F77373",
+"EA c #FFFFF7F75252",
+"FA c #FFFFF7F74242",
+"GA c #FFFFF7F73939",
+"HA c #FFFFEFEF8C8C",
+"IA c #FFFFEFEF4A4A",
+"JA c #FFFFEFEF2929",
+"KA c #F7F7E7E77B7B",
+"LA c #F7F7C6C6ADAD",
+"MA c #F7F7B5B59C9C",
+"NA c #F7F7ADAD9494",
+"OA c #EFEFF7F7F7F7",
+"PA c #EFEFF7F7EFEF",
+"AB c #EFEFEFEFEFEF",
+"BB c #EFEFEFEFDEDE",
+"CB c #EFEFE7E7A5A5",
+"DB c #EFEFDEDE7373",
+"EB c #EFEFDEDE3939",
+"FB c #EFEFDEDE2929",
+"GB c #EFEFD6D64242",
+"HB c #EFEFA5A58C8C",
+"IB c #EFEF94947B7B",
+"JB c #EFEF84847373",
+"KB c #EFEF84846363",
+"LB c #EFEF7B7B7373",
+"MB c #E7E7E7E7CECE",
+"NB c #E7E7E7E79494",
+"OB c #E7E7DEDE6B6B",
+"PB c #E7E7DEDE5252",
+"AC c #E7E7CECE5252",
+"BC c #E7E77B7B6363",
+"CC c #E7E773735A5A",
+"DC c #E7E76B6B5252",
+"EC c #E7E75A5A4A4A",
+"FC c #DEDEE7E7F7F7",
+"GC c #DEDEE7E7EFEF",
+"HC c #DEDEDEDEDEDE",
+"IC c #DEDEDEDEBDBD",
+"JC c #DEDECECE4A4A",
+"KC c #DEDE4A4A3939",
+"LC c #D6D6EFEFCECE",
+"MC c #D6D6DEDEEFEF",
+"NC c #D6D6DEDECECE",
+"OC c #CECEDEDEDEDE",
+"PC c #CECED6D6CECE",
+"AD c #CECECECEB5B5",
+"BD c #CECECECE5A5A",
+"CD c #C6C6E7E7C6C6",
+"DD c #C6C6DEDEEFEF",
+"ED c #C6C6D6D67373",
+"FD c #C6C6CECE4A4A",
+"GD c #BDBDDEDEB5B5",
+"HD c #BDBDCECEE7E7",
+"ID c #BDBDCECECECE",
+"JD c #BDBDC6C6D6D6",
+"KD c #BDBD39394242",
+"LD c #B5B5DEDEADAD",
+"MD c #B5B5BDBDCECE",
+"ND c #B5B5BDBD4242",
+"OD c #B5B5B5B59C9C",
+"PD c #ADADD6D6ADAD",
+"AE c #ADADCECEDEDE",
+"BE c #ADADBDBDB5B5",
+"CE c #ADADB5B54242",
+"DE c #ADAD4A4A5252",
+"EE c #A5A5BDBDDEDE",
+"FE c #A5A5BDBDCECE",
+"GE c #A5A5ADADB5B5",
+"HE c #A5A5ADAD4242",
+"IE c #A5A563637B7B",
+"JE c #9C9CD6D6A5A5",
+"KE c #9C9CBDBDD6D6",
+"LE c #9C9CBDBDBDBD",
+"ME c #9C9CADADCECE",
+"NE c #9C9CADADA5A5",
+"OE c #9C9C52525252",
+"PE c #9C9C4A4A5252",
+"AF c #9494CECE9C9C",
+"BF c #9494B5B5CECE",
+"CF c #9494ADAD5A5A",
+"DF c #9494ADAD3939",
+"EF c #8C8CCECE9494",
+"FF c #8C8CCECE8C8C",
+"GF c #8C8CB5B5B5B5",
+"HF c #8C8CADADD6D6",
+"IF c #8C8CADADCECE",
+"JF c #8C8CADADB5B5",
+"KF c #8C8CADADA5A5",
+"LF c #8C8C9C9CB5B5",
+"MF c #8C8C9C9CADAD",
+"NF c #8C8C8C8CA5A5",
+"OF c #8C8C52526363",
+"PF c #8C8C42425252",
+"AG c #8484B5B5CECE",
+"BG c #7B7BCECE9494",
+"CG c #7B7BC6C68484",
+"DG c #7B7BC6C67B7B",
+"EG c #7B7BADADC6C6",
+"FG c #7B7BADADADAD",
+"GG c #7B7B9C9CC6C6",
+"HG c #7B7B9C9CB5B5",
+"IG c #7B7B9C9CADAD",
+"JG c #7B7B84849C9C",
+"KG c #7B7B42425252",
+"LG c #737384848C8C",
+"MG c #737373738484",
+"NG c #737339395252",
+"OG c #6B6BC6C68484",
+"PG c #6B6BBDBD7B7B",
+"AH c #6B6BA5A5CECE",
+"BH c #6B6B9C9CBDBD",
+"CH c #6B6B9C9CA5A5",
+"DH c #6B6B94949C9C",
+"EH c #6B6B8C8CA5A5",
+"FH c #6B6B8C8C9C9C",
+"GH c #6B6B6B6B7B7B",
+"HH c #6B6B42425252",
+"IH c #6363BDBD6B6B",
+"JH c #6363B5B58C8C",
+"KH c #6363A5A58C8C",
+"LH c #63639C9CC6C6",
+"MH c #5A5AB5B56363",
+"NH c #5A5A9C9CBDBD",
+"OH c #5A5A8C8C9C9C",
+"PH c #5A5A7B7B8C8C",
+"AI c #5A5A6B6B8484",
+"BI c #5A5A63637B7B",
+"CI c #5252BDBD7373",
+"DI c #5252ADAD8484",
+"EI c #5252A5A57B7B",
+"FI c #4A4AB5B55A5A",
+"GI c #4A4A8C8CBDBD",
+"HI c #4A4A8C8CADAD",
+"II c #4A4A7B7B9C9C",
+"JI c #4A4A7B7B8484",
+"KI c #4A4A6B6B7B7B",
+"LI c #42429C9C7373",
+"MI c #42428C8C7B7B",
+"NI c #42425A5A7373",
+"OI c #42424A4A6B6B",
+"PI c #3939ADAD6B6B",
+"AJ c #3939ADAD5A5A",
+"BJ c #39399C9C6B6B",
+"CJ c #39398484BDBD",
+"DJ c #39397B7BADAD",
+"EJ c #39397B7B8C8C",
+"FJ c #393973737B7B",
+"GJ c #313163637B7B",
+"HJ c #31315A5A7B7B",
+"IJ c #292984846B6B",
+"JJ c #29296B6B7B7B",
+"KJ c #21217B7BB5B5",
+"LJ c #21217373A5A5",
+"MJ c #212173739C9C",
+"NJ c #21216B6B8C8C",
+"OJ c #212152527373",
+"PJ c #181884846363",
+"AK c #18186B6B7B7B",
+"BK c #18185A5A7373",
+"CK c #10106B6B9C9C",
+"DK c #10105A5A8484",
+"EK c #08087B7B6363",
+"FK c #08085A5A7B7B",
+"GK c #00006B6B9C9C",
+"HK c #000063637B7B",
+"IK c #00005A5A8C8C",
+/* pixels */
+"CJAHAHAHAHHILHLHBHLHLJNHNHNHGINHGINHGINHGINHGINHGINHGINHGINHGIKJ",
+"AHAAAAAAAGHDABBBGCBFBFHCPCOCEEHDJDEEJDEEHDJDEEHDJDEEJDEEHDEEJDNH",
+"AHAAAGAAABABABKEOCHCHCPCFEFEABAEHDFCDDJDDDMCJDHDFCHDHDGCDDJDDDLH",
+"EGAEDDABOAABOCEGHCHCPCHCBHJDJDEEJDEEJDEEJDAEJDEEAEJDEEJDEEJDEELH",
+"AHAAAAAAMCJDGCABHCHCBFOCPCJDEEJDEEJDEEAEEEEEAEJDEEFEFEFEEEEEJDGI",
+"AHAAAAPAAGGCABBBMCEGJDNCPCPCEEEEEEEEEEEEEEJDEEEEFEDIPIGFFEFEEENH",
+"EGOAKEAAABPAABKEHCHCHCHCMEBFABJDAEMCOCEEMCMCFEOCDDLEKHIDIDEEDDLH",
+"LHAEMCAAABABAEIFHCHCHCPCBHOCKEKEEEFEEEFEEEFEEEKEBFGEDIKFMEFEEEGI",
+"AHAAAAABGCJDABGCBBOCHGHCPCMDEEFEKEEEFEKEFEKEEEMEBFJFDIGEIFBFFEGI",
+"AHAAAAPAAHGCABBBMCJFMDOCPCJDMEKEKEMEKEKEKEKEMEIFGEFGEIIGJFGEMEGI",
+"AHAAAEAAABABGCNBDBOCNCNCMDEEMCEEEEMCJDKEDDDDLFCIDIEIPICIDIDIJDNH",
+"AHKEDDAAPAABOCKFPBICNCPCHGFEMEMEMEMEHFMEMEIFJFJELCCDCGBBCDLIIGHI",
+"EGAAAAABABJDMBNCPBADGENCMDMDMEHFIFHFMEHFIFMECHJEPCLDBGGDGDLIIGDJ",
+"AHAAAAAAAHEDKAOBEBBDCEODICMDIFIFMEIFIFIFIFIFCHEFLDPDOGGDPDMIFHHI",
+"AHAAOCABMBEABABAFABABAGBNELEHDKEHFDDEEIFJDEEFGCGLDPDPGPDPDAJMFHI",
+"AHDDEGABBBEABACAFABACBGBFHGEIFGGIFGGIFIFGGIFDHCGJEMHFIAFAFBJDHII",
+"AHAAAAABMBEAHAEAEADACABDODBEHGIFGGIFHGGGIFHGCHJHAFAFEFAFFFFJDHMJ",
+"AHAAAAAABEIADADAKACADAGBODNEGGHGIEIENFGGHGGGOHDGFFFFFFFFFFAJOHHI",
+"AHAAABABPCGAKADADAKADAFDODMFHIHIAIPFIIIIHIHIEJCIFFDGFFDGDGJJFJCK",
+"AHAAAHHCMBGAEAEAEAFAFAJCLGFKPEPEPEKDPEPEOIIKHKLIDGDGPGDGIHEKIKGK",
+"EGFCDDAANCJAIAIAIAIAIANDBKOJJBLALALBLAMANGFKHKFIMHPGMHFILIEKFKGK",
+"LHAAAABBNCFBJAJAJAEBGBDFBKOJBCNADCDCIBHBNGBKAKFIMHMHMHMHMHEKHKGK",
+"AHAAAAABADEBJCEBGBACACHEBKNIBCJBJBHBJBJBHHOJJJPIIHPGIHIHIHPJDKGK",
+"AHAAOCJDNCEBOBACOBOBOBHEEJNIKCLBKBBCKBCCKGHJJJMHDGIHPGDGIHEKEJCK",
+"AHAAKEABMBEBDBDBOBOBOBHENIAIKCDCDCDCDCECKGKIGJCIDGFFLGDGDGPJNJMJ",
+"CJAAOAABBBIADBHADBHAKAHEAIBIDCDCBCCCDCCCPFKIJIPGBGCFDGFFDGIJIIMJ",
+"AHAAAAABMBIABADBBAHAHAHEAIGHKCJBBCKBKBBCKGPHJIPGFFGFFFFFFFIJJILJ",
+"AHAAAAABMDFDFDBDFDNDBDHEFHJGDEDEDEDEOEDEOFPHMGBJBJAJBJBJBJMIOHDJ",
+"EGAAAAJDHGNFFHMGFHJGMGEHJGNFNFMGMGMGMGMGLGJGCHEHFHPHMGOHPHFHIGDJ",
+"LHAAOCIFIFIFLFLFMFMFJFLFIFJFHGIGMFIGNFMFMFLFIFLFHGIGFGIGIGHGJFCJ",
+"EGDDKEKEMEBFBFBFBFMEBFBFFEKEMEMEMEGEBFMEBFBFBFKEMEBFMEBFBFKEKEGI",
+"KJLHGINHGINHNHNHNHGINHNHGINHGINHNHGIGINHGINHNHGINHNHGIGINHNHGICJ"};
 
 //this icon has been taken from trolltech's Qt: qmessagebox.cpp
 static char *qtlogo_xpm[] = {
@@ -944,8 +1146,7 @@ BitMapQT::BitMapQT(GraphObj *g, QWidget *wi, int vr, int hr):anyOutput()
 	int w, h;
 
 	hres = (double)hr;		vres = (double)vr;
-	image = 0L;
-	hgo = 0L;
+	image = 0L;				hgo = 0L;
 	if(wi) {
 		w = wi->width();		h = wi->height();
 		}
@@ -968,8 +1169,8 @@ BitMapQT::BitMapQT(GraphObj *g, QWidget *wi, int vr, int hr):anyOutput()
 BitMapQT::BitMapQT(int w, int h, double hr, double vr):anyOutput()
 {
 	hres = hr;		vres = vr;
-	image = 0L;
-	hgo = 0L;
+	image = 0L;		hgo = 0L;
+	w = abs(w);		h = abs(h);
 	Box1.Xmin = Box1.Ymin = 0.0;
 	Box1.Xmax = w;					Box1.Ymax = h;
 	DeskRect.right = w;				DeskRect.bottom = h;
@@ -1072,7 +1273,7 @@ BitMapQT::oGetTextExtent(char *text, int cb, int *width, int *height)
 	if(!text) return false;
 	QRect rc = qPainter.boundingRect(0, 0, 10000, 1000, Qt::AlignLeft | Qt::AlignTop,
 		text, cb > 0 ? cb : strlen(text));
-	*width = rc.rRight() - rc.rLeft();		*height = rc.rBottom() - rc.rTop();
+	*width = rc.rRight() - rc.rLeft();		*height = TxtSet.iSize +2;
 	return true;
 }
 
@@ -1371,6 +1572,8 @@ OutputQT::UpdateRect(RECT *rc, bool invert)
 	else {
 		y1 = rc->bottom;	y2 = rc->top;
 		}
+	if(x2 > DeskRect.right) x2 = DeskRect.right;
+	if(y2 > DeskRect.bottom) y2 = DeskRect.right;
 	bitBlt(widget, x1, y1, mempic, x1, y1,
 		x2 - x1, y2 - y1, invert ? Qt::NotCopyROP : Qt::CopyROP);
 	return true;
@@ -1438,10 +1641,20 @@ OutputQT::SetMenu(int type)
 		file->insertItem("n.a.", widget, SLOT(cmFile5()), 0, CM_FILE5);
 		file->insertItem("n.a.", widget, SLOT(cmFile6()), 0, CM_FILE6);
 
+		QPopupMenu *insert = new QPopupMenu(widget);
+		insert->insertItem("&Rows", widget, SLOT(cmInsRow()));
+		insert->insertItem("&Columns", widget, SLOT(cmInsCol()));
+
+		QPopupMenu *Delete = new QPopupMenu(widget);
+		Delete->insertItem("&Rows", widget, SLOT(cmDelRow()));
+		Delete->insertItem("&Columns", widget, SLOT(cmDelCol()));
+
 		QPopupMenu *edit = new QPopupMenu(widget);
 		edit->insertItem("&Undo", widget, SLOT(cmUndo()), Qt::CTRL + Qt::Key_Z);
 		edit->insertSeparator();
 		edit->insertItem("&Rows/Cols", widget, SLOT(cmAddRowCol()));
+		edit->insertItem("&Insert", insert);
+		edit->insertItem("&Delete", Delete);
 		edit->insertSeparator();
 		edit->insertItem("&Copy", widget, SLOT(cmCopy()), Qt::CTRL + Qt::Key_C);
 		edit->insertItem("C&ut", widget, SLOT(cmCut()), Qt::CTRL + Qt::Key_X);
@@ -1696,11 +1909,12 @@ RLPwidget::RLPwidget(QWidget *par, const char *name, anyOutput *o, GraphObj *g)
 
 RLPwidget::~RLPwidget()
 {
+	if(OutputClass)((OutputQT*)OutputClass)->widget = 0L;
+	OutputClass = 0L;
 	if(BaseObj){
 		BaseObj->Command(CMD_CAN_DELETE, 0L, 0L);
 		BaseObj = 0L;
 		}
-	if(OutputClass)((OutputQT*)OutputClass)->widget = 0L;
 }
 
 //public slots: menu items, events
@@ -1738,7 +1952,13 @@ RLPwidget::cmSaveDataAs()
 void
 RLPwidget::cmExit()
 {
-	delete(this);
+	if(BaseObj) {
+		if(BaseObj->Command(CMD_CAN_CLOSE, 0L, 0L)){
+			BaseObj = 0L;
+			delete this;
+			}
+		}
+	else delete(this);
 }
 
 void
@@ -1793,8 +2013,8 @@ void
 RLPwidget::cmCopy()
 {
 	if(BaseObj && BaseObj->Id == GO_SPREADDATA) {
-		BaseObj->Command(CMD_QUERY_COPY, 0L, OutputClass);
-		CopyData(BaseObj);
+		if(BaseObj->Command(CMD_QUERY_COPY, 0L, OutputClass))
+			CopyData(BaseObj);
 		}
 }
 
@@ -1956,6 +2176,30 @@ RLPwidget::cmFillRange()
 	BaseObj->Command(CMD_FILLRANGE, 0L, OutputClass);
 }
 
+void
+RLPwidget::cmInsRow()
+{
+	BaseObj->Command(CMD_INSROW, 0L, OutputClass);
+}
+
+void
+RLPwidget::cmInsCol()
+{
+	BaseObj->Command(CMD_INSCOL, 0L, OutputClass);
+}
+
+void
+RLPwidget::cmDelRow()
+{
+	BaseObj->Command(CMD_DELROW, 0L, OutputClass);
+}
+
+void
+RLPwidget::cmDelCol()
+{
+	BaseObj->Command(CMD_DELCOL, 0L, OutputClass);
+}
+
 void ToolMenu(GraphObj *b, anyOutput *o, int tm)
 {
 	if(b && o) b->Command(CMD_TOOLMODE, (void*)(& tm), o);
@@ -2040,10 +2284,14 @@ void
 RLPwidget::closeEvent(QCloseEvent *e)
 {
 	if(BaseObj){
-		BaseObj->Command(CMD_CAN_DELETE, 0L, 0L);
-		BaseObj = 0L;
+		if(BaseObj->Command(CMD_CAN_CLOSE, 0L, 0L)) {
+			if(OutputClass)((OutputQT*)OutputClass)->widget = 0L;
+			OutputClass = 0L;
+			BaseObj = 0L;
+			e->accept();
+			}
 		}
-	e->accept();
+	else e->accept();
 }
 
 void
@@ -2057,41 +2305,57 @@ RLPwidget::mouseDoubleClickEvent(QMouseEvent *e)
 void
 RLPwidget::mousePressEvent(QMouseEvent *e)
 {
+	int i;
 	MouseEvent mev = {1, e->button() == Qt::LeftButton ? MOUSE_LBDOWN : -1, e->x(), e->y()};
 
 	HideTextCursor();
+	i = e->state();
+	if(i & Qt::ShiftButton) mev.StateFlags |= 0x08;
+	if(i & Qt::ControlButton) mev.StateFlags |= 0x10;
 	if (BaseObj)BaseObj->Command(CMD_MOUSE_EVENT, (void *)&mev, OutputClass);
 }
 
 void
 RLPwidget::mouseReleaseEvent(QMouseEvent *e)
 {
+	int i;
 	MouseEvent mev = {0, e->button() == Qt::LeftButton ? MOUSE_LBUP :
 		e->button() == Qt::RightButton ? MOUSE_RBUP : -1, e->x(), e->y()};
 
+	i = e->state();
+	if(i & Qt::ShiftButton) mev.StateFlags |= 0x08;
+	if(i & Qt::ControlButton) mev.StateFlags |= 0x10;
 	if (BaseObj) BaseObj->Command(CMD_MOUSE_EVENT, (void *)&mev, OutputClass);
 }
 
 void
 RLPwidget::mouseMoveEvent(QMouseEvent *e)
 {
+	int i;
 	MouseEvent mev = {0, MOUSE_MOVE, e->x(), e->y()};
 
-	if(e->state() == Qt::LeftButton) mev.StateFlags = 1;
+	i = e->state();
+	if(i & Qt::LeftButton) mev.StateFlags |= 0x01;
+	if(i & Qt::ShiftButton) mev.StateFlags |= 0x08;
+	if(i & Qt::ControlButton) mev.StateFlags |= 0x10;
 	if (BaseObj) BaseObj->Command(CMD_MOUSE_EVENT, (void *)&mev, OutputClass);
 }
 
 void
 RLPwidget::keyPressEvent(QKeyEvent *e)
 {
-	int c;
+	int i, c;
 
 	if(BaseObj) switch(c = e->key()) {
 		case Key_Prior:
-			BaseObj->Command(CMD_PAGEUP, 0L, OutputClass);
+			i = e->state();
+			if(i & Qt::ShiftButton) BaseObj->Command(CMD_SHPGUP, 0L, OutputClass);
+			else BaseObj->Command(CMD_PAGEUP, 0L, OutputClass);
 			break;
 		case Key_Next:
-			BaseObj->Command(CMD_PAGEDOWN, 0L, OutputClass);
+			i = e->state();
+			if(i & Qt::ShiftButton) BaseObj->Command(CMD_SHPGDOWN, 0L, OutputClass);
+			else BaseObj->Command(CMD_PAGEDOWN, 0L, OutputClass);
 			break;
 		case Key_Left:
 			if(e->state() & ShiftButton) BaseObj->Command(CMD_SHIFTLEFT, 0L, OutputClass);
@@ -2556,6 +2820,8 @@ DlgWidget::keyPressEvent(QKeyEvent *e)
 			c = e->ascii();
 			if(c >1 && c < 256)
 				if(c == 26) dlg->Command(CMD_UNDO, 0L, OutputClass);
+				else if(c == 0x03) dlg->Command(CMD_COPY, 0L, OutputClass);
+				else if(c == 0x16) dlg->Command(CMD_PASTE, 0L, OutputClass);
 				else dlg->Command(CMD_ADDCHAR, (void *)(& c), OutputClass);
 			break;
 		}
@@ -2610,7 +2876,7 @@ void *CreateDlgWnd(char *title, int x, int y, int width, int height, tag_DlgObj
 	else w->move(CurrWidgetPos.x+x, CurrWidgetPos.y+y);
 	o = new OutputQT(w);
 	o->units = defs.cUnits;
-	o->Erase(0x00c0c0c0L);
+	o->Erase(0x00e0e0e0L);
 	if(flags & 0x04) {
 		w->startTimer(100);
 		}
@@ -2626,6 +2892,7 @@ void LoopDlgWnd() 	//keep message processing running
 
 void CloseDlgWnd(void *hDlg)
 {
+	HideCopyMark();
 	if(hDlg) {
 		delete((DlgWidget*) hDlg);
 		}
