@@ -439,6 +439,43 @@ char **split(char *str, char sep, int *nl)
 	return ptr;
 }
 
+char *fit_num_rect(anyOutput *o, int max_width, char *num_str)
+{
+	int i, j, k, w, h, slen;
+	char mant[20], expo[20], fmt[10];
+	double num;
+
+	o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+	if(w < (max_width-5)) return num_str;
+	//first try to remove leading zero from exponent
+	for(i = 0; i < slen; i++) if(num_str[i] == 'e') break;
+	if(num_str[i] == 'e') while (num_str[i+2] == '0') {
+		for(j = i+2; num_str[j]; j++) num_str[j] = num_str[j+1];
+		o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+		if(w < (max_width-5)) return num_str;
+		}
+	//no success: reduce the number of digit by rounding
+	for(i = k = 0; i <= slen; i++){
+		if(num_str[i] == '.') k = i;
+		if((mant[i] = num_str[i]) == 'e' || mant[i] == 0) break;
+		}
+	if(num_str[i] =='e') mant[i] = 0;						k = i - k-1;
+	for(j = 0; num_str[i]; j++, i++) expo[j] = num_str[i];		expo[j] = 0;
+	sscanf(mant, "%lf", &num);	
+	if(k >0) do {
+		sprintf(fmt, "%s.%dlf%s", "%", k, expo);
+		sprintf(num_str, fmt, num);							k--;
+		o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+		if(w < (max_width-5)) return num_str;
+		}while (k >= 0);
+	//cannot fit: return hash marks instead
+	for(i = w = 0; w < (max_width-5); i++) {
+		sprintf(num_str+i, "#");
+		o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+		}
+	num_str[i-1] = 0;		return num_str;
+}
+
 //----------------------------------------------------------------------------
 // bounding rectangle utilities
 //----------------------------------------------------------------------------
@@ -936,6 +973,8 @@ void DeleteGO(GraphObj *go)
 	case GO_OBJTREE:		delete((ObjTree*)go);		break;
 	case GO_FREQDIST:		delete((FreqDist*)go);		break;
 	case GO_GRID3D:			delete((Grid3D*)go);		break;
+	case GO_FUNC3D:			delete((Func3D*)go);		break;
+	case GO_XYSTAT:			delete((xyStat*)go);		break;
 	default:
 		sprintf(TmpTxt, "Cannot delete Object\nwith Id %ld", go->Id);
 		ErrorBox(TmpTxt);
@@ -943,6 +982,35 @@ void DeleteGO(GraphObj *go)
 		}
 }
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+//Delete a graphic object from a list 
+bool DeleteGOL(GraphObj ***gol, long n, GraphObj *go, anyOutput *o)
+{
+	long i;
+	int c;
+	GraphObj **g, *p;
+
+	if(!gol || !(*gol) || !go || !n) return false;
+	for (i = 0, c = 0, g = *gol; i < n; i++, g++) {
+		if(*g) {
+			c++;
+			if(*g == go) {
+				p = (*g)->parent;
+				if(o) o->HideMark();
+				Undo.DeleteGO(g, 0L, o);
+				if(c == 1) {
+					for (g++, i++ ;i < n; i++, g++) {
+						if(*g) return true;
+						}
+					if(p) Undo.DropMemory(p, (void**) gol, UNDO_CONTINUE);
+					}
+				return true;
+				}
+			}
+		}
+	return false;
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //backup file before writing a new one
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1015,6 +1083,16 @@ bool FileExist(char *FileName)
 	return true;
 }
 
+
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+//check Object for certain properties
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+bool IsPlot3D(GraphObj *g)
+{
+	if(g && (g->Id == GO_PLOT3D || g->Id == GO_FUNC3D)) return true;
+	return false;
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //duplicate a block of memory
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1069,6 +1147,7 @@ unsigned int HashValue(unsigned char *str)
 		if(str[i] > 32) ret = ((str[i]-32) + (ret <<2));
 		i++;
 		}while(str[i]);
+	if(i < 4) memcpy(&ret, str, i);
 	return ret;
 }
 
