@@ -242,6 +242,19 @@ bool YesNoBox(char *Msg)
 	return false;
 }
 
+int YesNoCancelBox(char *Msg)
+{
+	int res;
+
+	res = MessageBox(0, Msg, "RLPlot", MB_YESNOCANCEL | MB_ICONQUESTION);
+	switch(res) {
+	case IDYES: return 1;
+	case IDNO:	return 0;
+	default:	return 2;
+		}
+	return 0;
+}
+
 void Qt_Box()
 {
 	MessageBox(0, "No Qt installed\nunder Windows", "Error", MB_OK | MB_ICONQUESTION);
@@ -437,6 +450,39 @@ void EmptyClip()
 	CloseClipboard();
 }
 
+void CopyText(char *txt, int len)
+{
+	HGLOBAL hmem;
+	unsigned char* buf;
+
+	if(!txt || !txt[0]) return;
+	if(!len) len = strlen(txt);
+	OpenClipboard(MainWnd);
+	EmptyClipboard();
+	if(hmem = GlobalAlloc(GMEM_MOVEABLE, len+2)) {
+		if(buf = (unsigned char *)GlobalLock(hmem)) {
+			memcpy(buf, txt, len);	buf[len] = 0;
+			GlobalUnlock(hmem);
+			SetClipboardData(CF_TEXT, hmem);
+			}
+		}
+	CloseClipboard();
+}
+
+unsigned char* PasteText()
+{
+	HANDLE hmem = 0;
+	unsigned char *ptr, *ret=0L;
+	
+	OpenClipboard(MainWnd);
+	if((hmem = GetClipboardData(CF_TEXT)) && (ptr = (unsigned char*) GlobalLock(hmem))){
+		ret = (unsigned char*) strdup((char*)ptr);
+		}
+	if(hmem) GlobalUnlock(hmem);
+	CloseClipboard();
+	return ret;
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Get display (desktop) size
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -822,8 +868,7 @@ BitMapWin::oGetTextExtent(char *text, int cb, int *width, int *height)
 		d = (TextExtent.cx * ((7.0 + d)/8.0));
 		TextExtent.cx = iround(d);
 		}
-	*width = TextExtent.cx;
-	*height = TextExtent.cy;
+	*width = TextExtent.cx;					*height = TextExtent.cy;
 	return true;
 }
 
@@ -1707,31 +1752,6 @@ void FindBrowser()
 	ReleaseDC(hwnd, dc);
 }
 
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Shutdown or reboot windows
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-void HardExit(bool reboot)
-{
-	int tpsize;
-	HANDLE hProcess, hAccessToken;
-	TOKEN_PRIVILEGES *tp;
-
-	hProcess = GetCurrentProcess();
-	tpsize = sizeof(TOKEN_PRIVILEGES) + sizeof(LUID_AND_ATTRIBUTES);
-	tp = (TOKEN_PRIVILEGES *)malloc(tpsize);
-	if(tp){
-		tp->PrivilegeCount = 1;
-		tp->Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
-		if(LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tp->Privileges[0].Luid)) {
-			if(OpenProcessToken(hProcess, TOKEN_ADJUST_PRIVILEGES, &hAccessToken))
-				AdjustTokenPrivileges(hAccessToken, FALSE, tp, tpsize, NULL, NULL);
-			}
-		free(tp);
-		}
-	if(reboot) ExitWindowsEx(EWX_REBOOT | EWX_FORCE, NULL);
-	else ExitWindowsEx(EWX_POWEROFF | EWX_FORCE, NULL);
-}
-
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Windos entry point
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1746,6 +1766,14 @@ WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrevInstance,
 	dlgtxtheight = 16;
 
 	if(lpCmdLine && lpCmdLine[0] && FileExist(lpCmdLine)) LoadFile = strdup(lpCmdLine);
+	else if(lpCmdLine) {								//probably Unicode 
+		sprintf(TmpTxt, "%s", lpCmdLine);
+		if(TmpTxt[0] == '"'){
+			strcpy(TmpTxt, TmpTxt+1);
+			if(TmpTxt[0]) TmpTxt[strlen(TmpTxt)-1] = 0;
+			}
+		if(TmpTxt[0]) LoadFile= strdup(TmpTxt);
+		}
 	hInstance = hInst;
 	wndclass.style = CS_BYTEALIGNWINDOW | CS_DBLCLKS;
 	wndclass.lpfnWndProc = WndProc;
@@ -1848,15 +1876,20 @@ void ScrollEvent(bool bVert, HWND hwnd, UINT type, GraphObj *g, OutputWin *w)
 			pos = si.nPos + LineStep;
 			break;
 		case SB_PAGEUP:
-			pos = (si.nPos - (int)si.nPage) >= si.nMin ?
-				(si.nPos - si.nPage) : si.nMin;
+			if(g->Id == GO_SPREADDATA) {
+				g->Command(CMD_PAGEUP, 0L, w);
+				return;
+				}
+			pos = (si.nPos - (int)si.nPage) >= si.nMin ? (si.nPos - si.nPage) : si.nMin;
 			break;
 		case SB_PAGEDOWN:
-			pos = ((unsigned)si.nPos + si.nPage*2) < (unsigned)si.nMax ?
-				(si.nPos + si.nPage) : (si.nMax - si.nPage+1);
+			if(g->Id == GO_SPREADDATA) {
+				g->Command(CMD_PAGEDOWN, 0L, w);
+				return;
+				}
+			pos = (si.nPos + (int)si.nPage*2) < si.nMax ? (si.nPos + si.nPage) : (si.nMax - si.nPage+1);
 			break;
-		case SB_THUMBTRACK:
-		case SB_THUMBPOSITION:
+		case SB_THUMBTRACK:		case SB_THUMBPOSITION:
 			pos = si.nTrackPos;
 			break;
 		default:
@@ -1977,20 +2010,17 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 				if(CopyWMF && CopyWMF->StartPage()) {
 					copy_obj->DoPlot(CopyWMF);
 					CopyWMF->EndPage();
+					delete CopyWMF;					CopyWMF = NULL;
 					}
-				delete CopyWMF;
-				CopyWMF = NULL;
 				if(copy_obj->Id == GO_GRAPH || copy_obj->Id == GO_PAGE) copy_obj->DoPlot(0L);
 				break;
 			case CF_BITMAP:
 				if((CopyBMP = new BitMapWin(copy_obj)) && CopyBMP->StartPage()) {
-					copy_obj->DoPlot(CopyBMP);
-					CopyBMP->EndPage();
+					copy_obj->DoPlot(CopyBMP);			CopyBMP->EndPage();
 					SetClipboardData(CF_BITMAP, CopyBMP->scr);
-					CopyBMP->scr = 0L;
+					CopyBMP->scr = 0L;					CopyBMP->go = 0L;
+					delete CopyBMP;						CopyBMP = NULL;
 					}
-				delete CopyBMP;
-				CopyBMP = NULL;
 				if(copy_obj->Id == GO_GRAPH || copy_obj->Id == GO_PAGE) copy_obj->DoPlot(0L);
 				break;
 			case CF_SYLK:		case CF_TEXT:
@@ -2007,11 +2037,12 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		wParam &= 0xffff;
 		if(g && w) switch(wParam) {
 		case CM_EXIT:
-			g->Command(CMD_CAN_DELETE, 0L, 0L);
-			SetWindowLong(hwnd, 0, 0L);
-			SetWindowLong(hwnd, GWL_USERDATA, 0L);
-			w->go = 0L;
-			DestroyWindow(hwnd);
+			if(g->Command(CMD_CAN_CLOSE, 0L, 0L)) {
+				SetWindowLong(hwnd, 0, 0L);
+				SetWindowLong(hwnd, GWL_USERDATA, 0L);
+				w->go = 0L;
+				DestroyWindow(hwnd);
+				}
 			return 0;
 		case CM_PASTE:
 			w->MouseCursor(MC_WAIT, true);
@@ -2134,12 +2165,6 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		case CM_ADDROWCOL:
 			g->Command(CMD_ADDROWCOL, (void *)NULL, w);
 			return 0;
-		case CM_REBOOT:
-			HardExit(true);
-			return 0;
-		case CM_SHUTDOWN:
-			HardExit(false);
-			return 0;
 		case CM_DEFAULTS:
 			g->Command(CMD_CONFIG, 0L, w);
 			return 0;
@@ -2186,7 +2211,7 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		case CM_UPARRKEY:   case CM_DOWNARRKEY:		case CM_POS_FIRST:
 		case CM_POS_LAST:	case CM_SHLEFT:			case CM_SHRIGHT:
 		case CM_SHUP:		case CM_SHDOWN:			case CM_TAB:
-		case CM_SHTAB:
+		case CM_SHTAB:		case CM_SHPGUP:			case CM_SHPGDOWN:
 			switch(wParam) {
 			case CM_DELKEY:			cc = CMD_DELETE;		break;
 			case CM_LEFTARRKEY:		cc = CMD_CURRLEFT;		break;
@@ -2201,6 +2226,8 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 			case CM_SHDOWN:			cc = CMD_SHIFTDOWN;		break;
 			case CM_TAB:			cc = CMD_TAB;			break;
 			case CM_SHTAB:			cc = CMD_SHTAB;			break;
+			case CM_SHPGUP:			cc = CMD_SHPGUP;		break;
+			case CM_SHPGDOWN:		cc = CMD_SHPGDOWN;		break;
 				}
 			g->Command(cc, (void *)NULL, w);
 			return 0;
@@ -2210,6 +2237,18 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		case CM_UNDO:
 			g->Command(CMD_UNDO, 0L, w);
 			return 0;
+		case CM_INSROW:
+			g->Command(CMD_INSROW, 0L, w);
+			return 0;
+		case CM_INSCOL:
+			g->Command(CMD_INSCOL, 0L, w);
+			return 0;
+		case CM_DELROW:
+			g->Command(CMD_DELROW, 0L, w);
+			return 0;
+		case CM_DELCOL:
+			g->Command(CMD_DELCOL, 0L, w);
+			return 0;
 		default:
 			sprintf(TmpTxt, "Command 0x%x (%d)\nreceived", wParam & 0xffff,
 				wParam & 0xffff);
@@ -2217,13 +2256,13 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		}
 		return 0;
 	case WM_CLOSE:
-		if(g) {
-			g->Command(CMD_CAN_DELETE, 0L, 0L);
+		if(g && g->Command(CMD_CAN_CLOSE, 0L, 0L)) {
 			SetWindowLong(hwnd, 0, 0L);
 			SetWindowLong(hwnd, GWL_USERDATA, 0L);
 			w->go = 0L;
+			DestroyWindow(hwnd);
 			}
-		DestroyWindow(hwnd);
+		else if(!g) DestroyWindow(hwnd);
 		return 0;
 	case WM_DESTROY:
 		if(hwnd == MainWnd)PostQuitMessage(0);
@@ -2276,6 +2315,8 @@ long FAR PASCAL DlgWndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		break;
 	case WM_CHAR:
 		if(0x09 == (cc = wParam & 0xff)) break;		//ignore Tab
+		if(cc == 0x03) return d->Command(CMD_COPY, 0L, w);	//^c copy
+		if(cc == 0x16) return d->Command(CMD_PASTE, 0L, w);	//^v paste
 		if(d && w) d->Command(CMD_ADDCHAR, (void *)(& cc), w);
 		break;
 	case WM_LBUTTONDOWN:	case WM_LBUTTONDBLCLK:
@@ -2355,7 +2396,7 @@ void *CreateDlgWnd(char *title, int x, int y, int width, int height, tag_DlgObj
 		x, y, width, height, GetFocus(), NULL, hInstance, NULL);
 	w = new OutputWin(0L, hDlg);
 	w->units = defs.cUnits;
-	if(hDlg && w && w->Erase(0x00c0c0c0L)) {
+	if(hDlg && w && w->Erase(0x00e0e0e0L)) {
 		SetWindowLong(hDlg, GWL_USERDATA, (long)w);
 		SetWindowLong(hDlg, 0, (long)d);
 		ShowWindow(hDlg, SW_SHOW);
@@ -2387,6 +2428,7 @@ void LoopDlgWnd() 	//keep message processing running
 
 void CloseDlgWnd(void *hDlg)
 {
+	HideCopyMark();
 	if(hDlg) SendMessage((HWND)hDlg, WM_CLOSE, 0, 0);
 }
 
