@@ -26,13 +26,20 @@
 inline int iround(double a) {return a < 0.0 ?(int)(a-0.499) : (int)(a+0.499);}
 inline int WriteFloatToBuff(char *buff, double val) {return sprintf(buff, " %g", val);}
 
+#define _PI		3.1415926535897932384626433832795028841971693993751
+#define _SQRT2	1.4142135623730950488016887242096980785696718753769
+
 #ifdef _WINDOWS
 #include <windows.h>
 #define w_char unsigned short
+#define _SBINC	1				//scrollbox extra space/line
+#define _TXH	4.0				//graph text default size in mm
 
 #else
 #define DWORD unsigned long
 #define w_char unsigned short
+#define _SBINC	8				//scrollbox extra space/line
+#define _TXH	3.0				//graph text default size in mm
 
 typedef struct tagPOINT { // pt 
     long x; 
@@ -69,7 +76,7 @@ enum {SIZE_MINE, SIZE_SYMBOL, SIZE_SYM_LINE, SIZE_DATA_LINE, SIZE_TEXT,
 	SIZE_A, SIZE_B, SIZE_MX, SIZE_MY, SIZE_MIN_Z, SIZE_MAX_Z, SIZE_MIN_X, SIZE_MAX_X,
 	SIZE_MIN_Y, SIZE_MAX_Y, SIZE_TICK_ANGLE, SIZE_RRECT_RAD, SIZE_XCENT, SIZE_YCENT,
 	SIZE_ZCENT, SIZE_DRADIUS, SIZE_CURSORPOS, SIZE_CURSOR_XMIN, SIZE_CURSOR_YMIN,
-	SIZE_CURSOR_XMAX, SIZE_CURSOR_YMAX, SIZE_XSTEP};
+	SIZE_CURSOR_XMAX, SIZE_CURSOR_YMAX, SIZE_XSTEP, SIZE_LSPC};
 enum {COL_SYM_LINE, COL_SYM_FILL, COL_DATA_LINE, COL_TEXT, COL_BG,
 	COL_AXIS, COL_BAR_LINE, COL_BAR_FILL, COL_ERROR_LINE, COL_BUBBLE_LINE, 
 	COL_BUBBLE_FILLLINE, COL_BUBBLE_FILL, COL_ARROW, COL_WHISKER, COL_BOX_LINE,
@@ -89,12 +96,12 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELETE,
 	CMD_BUBBLE_ATTRIB, CMD_BUBBLE_FILL, CMD_BUBBLE_LINE, CMD_DL_LINE, 
 	CMD_DL_TYPE, CMD_SEG_FILL, CMD_SEG_LINE, CMD_SELECT, CMD_MOVE, CMD_CUT,
 	CMD_SETSCROLL, CMD_SETHPOS, CMD_SETVPOS, CMD_PG_FILL, CMD_BOUNDS,
-	CMD_SHIFT_OUT, CMD_CAN_DELETE, CMD_RESET_LINE, CMD_SET_TICKSTYLE,
+	CMD_SHIFT_OUT, CMD_CAN_DELETE, CMD_CAN_CLOSE, CMD_RESET_LINE, CMD_SET_TICKSTYLE,
 	CMD_GET_GRIDLINE, CMD_SET_GRIDLINE, CMD_SET_GRIDTYPE, CMD_TLB_TXTDEF,
 	CMD_DROP_LABEL, CMD_DROP_OBJECT, CMD_PAGEUP, CMD_PAGEDOWN, CMD_AUTOSCALE,
 	CMD_MRK_DIRTY, CMD_SETNAME, CMD_TOOLMODE, CMD_DROPFILE, CMD_AXIS, CMD_INIT,
 	CMD_GET_CELLDIMS, CMD_SET_CELLDIMS, CMD_TEXTSIZE, CMD_PASTE_TSV, CMD_PASTE_XML,
-	CMD_PASTE_CSV, CMD_COPY_TSV, CMD_COPY_XML, CMD_COPY_SYLK, CMD_QUERY_COPY,
+	CMD_PASTE_SSV, CMD_PASTE_CSV, CMD_COPY_TSV, CMD_COPY_XML, CMD_COPY_SYLK, CMD_QUERY_COPY,
 	CMD_MOUSECURSOR, CMD_NEWPAGE, CMD_MINRC, CMD_MAXRC,CMD_SETCHILD, CMD_SYM_FILL,
 	CMD_LINEUP, CMD_LINEDOWN, CMD_CONFIG, CMD_FINDTEXT, CMD_MOVE_TOP, CMD_MOVE_UP,
 	CMD_MOVE_DOWN, CMD_MOVE_BOTTOM, CMD_UPDATE, CMD_CURRPOS, CMD_POS_FIRST, CMD_POS_LAST,
@@ -103,10 +110,11 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELETE,
 	CMD_ZOOM, CMD_CLIP, CMD_STARTLINE, CMD_ADDTOLINE, CMD_REQ_POINT, CMD_DRAW_LATER,
 	CMD_SEG_MOVEABLE, CMD_ARROW_ORG3D, CMD_MUTATE, CMD_PRINT, CMD_UPDHISTORY, CMD_ALLTEXT,
 	CMD_SET_LINE, CMD_SAVE_SYMBOLS, CMD_SAVE_TICKS, CMD_SAVE_BARS, CMD_SAVE_BARS_CONT,
-	CMD_SAVE_ERRS, CMD_SAVE_ARROWS, CMD_SAVE_DROPLINES, CMD_UNLOCK, CMD_SYMTEXT_UNDO,
+	CMD_SAVE_ERRS, CMD_SAVE_ARROWS, CMD_SAVE_DROPLINES, CMD_SAVE_LABELS, CMD_UNLOCK, CMD_SYMTEXT_UNDO,
 	CMD_FILLRANGE, CMD_BUSY, CMD_ERROR, CMD_CLEAR_ERROR, CMD_SETPARAM, CMD_SETFUNC,
 	CMD_HIDE_MARK, CMD_LEGEND, CMD_FILENAME, CMD_LAYERS, CMD_OBJTREE, CMD_TEXTDEF,
-	CMD_HASSTACK, CMD_WRITE_GRAPHS, CMD_SETFONT, CMD_SETSTYLE};
+	CMD_HASSTACK, CMD_WRITE_GRAPHS, CMD_SETFONT, CMD_SETSTYLE, CMD_COPY, CMD_PASTE,
+	CMD_INSROW, CMD_INSCOL, CMD_DELROW, CMD_DELCOL, CMD_ADDTXT, CMD_ETRACC, CMD_SHPGUP, CMD_SHPGDOWN};
 enum {SYM_CIRCLE, SYM_CIRCLEF, SYM_RECT, SYM_RECTF, SYM_TRIAU, SYM_TRIAUF,
 	SYM_TRIAD, SYM_TRIADF, SYM_DIAMOND, SYM_DIAMONDF, SYM_PLUS, SYM_CROSS,
 	SYM_STAR, SYM_HLINE, SYM_VLINE, SYM_TEXT, SYM_POS_PARENT = 0x1000};
@@ -123,7 +131,8 @@ enum {GO_UNKNOWN, GO_AXIS, GO_TICK, GO_GRIDLINE, GO_SYMBOL, GO_BUBBLE, GO_BAR,
 	GO_PLOT = 0x100, GO_PLOTSCATT, GO_REGRESSION, GO_BARCHART, GO_BUBBLEPLOT, 
 	GO_BOXPLOT, GO_DENSDISP, GO_STACKBAR, GO_STACKPG, GO_WATERFALL, GO_POLARPLOT,
 	GO_PIECHART, GO_RINGCHART, GO_GROUP, GO_STARCHART, GO_SCATT3D, GO_PLOT3D,
-	GO_RIBBON, GO_LIMITS, GO_FUNCTION, GO_FITFUNC, GO_FREQDIST, GO_GRID3D,
+	GO_RIBBON, GO_LIMITS, GO_FUNCTION, GO_FITFUNC, GO_FREQDIST, GO_GRID3D, GO_FUNC3D,
+	GO_XYSTAT,
 	GO_GRAPH = 0x200, GO_PAGE, GO_SPREADDATA = 0x300, GO_DEFRW};
 enum {FILL_NONE, FILL_HLINES, FILL_VLINES, FILL_HVCROSS, FILL_DLINEU, FILL_DLINED,
 	FILL_DCROSS, FILL_STIPPLE1, FILL_STIPPLE2, FILL_STIPPLE3, FILL_STIPPLE4, 
@@ -145,7 +154,7 @@ enum {MENU_NONE, MENU_SPREAD, MENU_GRAPH, MENU_PAGE};
 enum {GT_UNKNOWN, GT_STANDARD, GT_CIRCCHART, GT_POLARPLOT, GT_3D};
 enum {ICO_NONE, ICO_INFO, ICO_ERROR, ICO_RLPLOT, ICO_QT};
 enum {FF_UNKNOWN, FF_CSV, FF_TSV, FF_XML, FF_SYLK, FF_RLP, FF_SVG, FF_EPS,
-	FF_WMF, FF_RLW};
+	FF_WMF, FF_RLW, FF_SSV};
 enum {LB_X_DATA = 0x01, LB_X_PARENT = 0x02, LB_Y_DATA = 0x10, LB_Y_PARENT = 0x20};
 enum {BUBBLE_CIRCLE = 0x000, BUBBLE_SQUARE = 0x001, BUBBLE_UPTRIA = 0x002,
 	BUBBLE_DOWNTRIA = 0x003, BUBBLE_UNITS = 0x000, BUBBLE_XAXIS = 0x010,
@@ -153,6 +162,7 @@ enum {BUBBLE_CIRCLE = 0x000, BUBBLE_SQUARE = 0x001, BUBBLE_UPTRIA = 0x002,
 	BUBBLE_AREA = 0x200};
 enum {DH_UNKNOWN, DH_12, DH_22, DH_19, DH_29, DH_39, DH_49, DH_59, DH_69, DH_79,
 	DH_89, DH_99, DH_18, DH_28, DH_38, DH_48, DH_58, DH_68, DH_78, DH_88, DH_DATA};
+enum {FE_NONE = 0x1000, FE_PARENT, FE_PLOT, FE_FLUSH, FE_DELOBJ, FE_REPLGO, FE_MUTATE};
 
 //drop line styles
 #define DL_LEFT          0x001
@@ -315,6 +325,8 @@ public:
 	int CountItems();
 	bool GetFirst(int *x, int *y);
 	bool GetNext(int *x, int *y);
+	bool NextRow(int *y);
+	bool NextCol(int *x);
 	bool IsInRange(int x, int y);
 	bool BoundRec(RECT *rec);
 
@@ -386,6 +398,7 @@ public:
 	double fiy2un(double fiy);
 	virtual bool SetLine(LineDEF *lDef){return false;};
 	bool GetLine(LineDEF *lDef);
+	virtual void Focus() {return;};
 	virtual void Caption(char *txt){return;};
 	virtual void MouseCursor(int cid, bool force){return;};
 	virtual bool SetScroll(bool, int, int, int, int){return false;};
@@ -427,12 +440,13 @@ enum {ET_UNKNOWN, ET_VALUE, ET_TEXT, ET_FORMULA, ET_ERROR, ET_BUSY=0x100,
 
 class EditText {
 public:
-	char *text;
-	int Align, type;
+	char *text, *ftext;
+	int Align, type, row, col;
 	void *parent;				//points to a data object: defined below
+	anyOutput *disp;
+	double Value;
 
-	EditText(void *par, POINT where, POINT right, char *msg);
-	EditText(void *par, char *msg);
+	EditText(void *par, char *msg, int r, int c);
 	~EditText();
 	bool AddChar(int c, anyOutput *Out, void *data_obj);
 	void Update(int select, anyOutput *Out, POINT *MousePos);
@@ -441,7 +455,7 @@ public:
 	bool Command(int cmd, anyOutput *Out, void *data_obj);
 	bool GetValue(double *v);
 	bool GetText(char *t, int size);
-	bool GetResult(anyResult *r);
+	bool GetResult(anyResult *r, bool use_last = false);
 	bool SetValue(double v);
 	bool SetText(char *t);
 	bool GetItem(char *text, int size);
@@ -452,6 +466,7 @@ public:
 	bool isValue();
 	bool isFormula();
 	bool isInRect(POINT *p) {return (p->x>loc.x && p->x<crb.x && p->y>loc.y && p->y<rb.y);};
+	bool hasMark() {return (m1 != m2 && m1 >= 0 && m2 >= 0);};
 
 private:
 	LineDEF *bgLine;
@@ -459,7 +474,6 @@ private:
 	int length, CursorPos, m1, m2, mx1, mx2;
 	POINT loc, rb, crb;
 	DWORD TextCol;
-	double Value;
 
 	void FindType();
 };
@@ -508,7 +522,7 @@ public:
 	virtual bool GetValue(int row, int col, double *v);
 	virtual bool GetText(int row, int col, char *txt, int len);
 	char ** GetTextPtr(int row, int col);
-	virtual bool GetResult(anyResult *r, int row, int col);
+	virtual bool GetResult(anyResult *r, int row, int col, bool use_last = false);
 	virtual bool GetSize(int *width, int *height);
 	virtual bool Select(POINT *p){return false;};
 	virtual bool WriteData(char *FileName) {return false;};
@@ -522,13 +536,14 @@ public:
 
 class StrData {
 public:
-	StrData(DataObj *par);
+	StrData(DataObj *par, RECT *rc = 0L);
 	~StrData();
 	bool GetSize(int *w, int *h);
 	void StrData::RestoreData(DataObj *dest);
 
 private:
-	int pw, ph, h, w;
+	int pw, ph;
+	RECT drc;
 	DataObj *src;
 	char ***str_data;
 };
@@ -622,7 +637,7 @@ public:
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 
 private:
-	bool bLBdown;
+	bool bLBdown, bSelected;
 	TextDEF TextDef;
 	LineDEF Line;
 	FillDEF Fill;
@@ -748,6 +763,8 @@ public:
 	bool FileIO(int rw);
 
 private:
+	bool DoAutoscale(anyOutput *o);
+
 	lfPOINT fPos;
 	double fs;
 	LineDEF BubbleLine, BubbleFillLine;
@@ -767,11 +784,14 @@ public:
 	bool SetSize(int select, double value);
 	bool SetColor(int select, DWORD col);
 	void DoPlot(anyOutput *target);
+	void DoMark(anyOutput *o, bool mark);
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	bool PropertyDlg();
 	bool FileIO(int rw);
 
 private:
+	anyOutput *mo;
+	RECT mrc;
 	double size;
 	LineDEF BarLine, HatchLine;
 	FillDEF BarFill;
@@ -807,6 +827,7 @@ public:
 	void FileValues(char *name, int type, double start, double step);
 	void SetValues();
 	void LineData(lfPOINT *val, long nval);
+	void DrawSpline(anyOutput *target);
 };
 
 class DataPolygon:public DataLine{
@@ -1208,7 +1229,10 @@ public:
 	LineDEF Line;
 
 	Line3D(GraphObj *par, DataObj *d, char *rx, char *ry, char *rz);
-	Line3D::Line3D(GraphObj *par, DataObj *d, fPOINT3D *pt, int n_pt);
+	Line3D(GraphObj *par, DataObj *d, fPOINT3D *pt, int n_pt, int xc1 = -1,
+		int xr1 = -1, int yc1 = -1, int yr1 = -1, int zc1 = -1, int zr1 = -1, 
+		int xc2 = -1, int xr2 = -1, int yc2 = -1, int yr2 = -1, int zc2 = -1, 
+		int zr2 = -1);
 	Line3D(int src);
 	~Line3D();
 	void DoPlot(anyOutput *o);
@@ -1221,8 +1245,10 @@ private:
 	line_segment **ls;
 	fPOINT3D *values, min, max;
 	POINT *pts;
-	char *x_range, *y_range, *z_range;
 	long nPts, npts;
+	char *x_range, *y_range, *z_range;
+	POINT *ssRef;
+	long cssRef;
 	bool bModified;
 	anyOutput *mo;
 	RECT mrc;
@@ -1289,7 +1315,7 @@ public:
 
 private:
 	lfPOINT fPos, fDist, cPos, cPos1, dist;
-	double si, csi, curr_z;
+	double si, csi, curr_z, lspc;
 	DWORD flags, undo_flags;
 	TextDEF TextDef;
 	long nLines;
@@ -1480,6 +1506,16 @@ public:
 
 class PlotScatt:public Plot{
 public:
+	char *xRange, *yRange;
+	long nPoints;
+	int DefSym;
+	lfPOINT BarDist;
+	Bar **Bars;
+	Symbol **Symbols;
+	DataLine *TheLine;
+	ErrorBar **Errors;
+	Label **Labels;
+
 	PlotScatt(GraphObj *par, DataObj *d, DWORD presel);
 	PlotScatt(GraphObj *par, DataObj *d, int cBars, Bar **bars);
 	PlotScatt(GraphObj *par, DataObj *d, int nPts, Symbol **sym, DataLine *lin);
@@ -1489,29 +1525,38 @@ public:
 	bool SetSize(int select, double value);
 	bool SetColor(int select, DWORD col);
 	void DoPlot(anyOutput *target);
-	bool Command(int cmd, void *tmpl, anyOutput *o);
+	virtual bool Command(int cmd, void *tmpl, anyOutput *o);
 	virtual bool PropertyDlg();
 	void RegGO(void *n);
-	bool FileIO(int rw);
+	virtual bool FileIO(int rw);
+
+	bool ForEach(int cmd, void *tmp, anyOutput *o);
 
 private:
 	DWORD DefSel;
-	lfPOINT BarDist;
-	char *xRange, *yRange, *ErrRange, *LbRange;
-	int DefSym;
-	long nPoints;
-	Bar **Bars;
-	Symbol **Symbols;
-	DataLine *TheLine;
-	ErrorBar **Errors;
+	char *ErrRange, *LbRange;
 	Arrow **Arrows;
 	DropLine **DropLines;
-	Label **Labels;
 
 	bool CreateBarChart();
-	enum {FE_NONE = 0x1000, FE_PARENT, FE_PLOT, FE_FLUSH, FE_DELOBJ, 
-		FE_REPLGO, FE_MUTATE};
-	bool ForEach(int cmd, void *tmp, anyOutput *o);
+};
+
+class xyStat:public PlotScatt{
+public:
+	xyStat(GraphObj *par, DataObj *d);
+	xyStat(int src);
+	~xyStat();
+	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool PropertyDlg();
+	virtual bool FileIO(int rw);
+
+	void CreateData();
+
+private:
+	double ci;
+	DataObj *curr_data;
+	char *case_prefix;
+
 };
 
 class BarChart:public PlotScatt{
@@ -1626,12 +1671,19 @@ public:
 	void RegGO(void *n);
 	bool FileIO(int rw);
 
+	bool ForEach(int cmd, void *tmp, anyOutput *o);
+	void CreateData();
+
 private:
+	char *xRange, *yRange, *case_prefix;
 	long nPoints;
+	double ci_box, ci_err;
+	DataObj *curr_data;
 	lfPOINT BoxDist;
 	Box **Boxes;
 	Whisker **Whiskers;
 	Symbol **Symbols;
+	Label **Labels;
 	DataLine *TheLine;
 };
 
@@ -1722,7 +1774,7 @@ public:
 	void DoUpdate();
 
 private:
-	int nPts;
+	long nPts;
 	char *ssRefA, *ssRefR;
 	lfPOINT CtDef;
 	double FacRad;
@@ -1787,9 +1839,11 @@ private:
 
 class Grid3D:public Plot {
 public:
-	Grid3D(GraphObj *par, DataObj *d);
+	Grid3D(GraphObj *par, DataObj *d, int sel, double x1=0.0, double xstep=1.0, double z1=0.0, double zstep=1.0);
 	Grid3D(int src);
 	~Grid3D();
+	bool SetSize(int select, double value);
+	bool SetColor(int select, DWORD col);
 	void DoPlot(anyOutput *o);
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	void RegGO(void *n);
@@ -1798,10 +1852,12 @@ public:
 	void CreateObs();
 
 private:
-	long nLines;
+	long nLines, nPlanes;
 	fPOINT3D start, step;
 	LineDEF Line;
+	FillDEF Fill;
 	Line3D **lines;
+	Plane3D **planes;
 };
 
 class Scatt3D:public Plot{
@@ -2024,6 +2080,7 @@ public:
 	Axis **Axes;
 	GraphObj **plots;
 	double *RotDef;
+	fPOINT3D cub1, cub2, rotC;
 
 	Plot3D(GraphObj *par, DataObj *d, DWORD flags);
 	Plot3D(int src);
@@ -2032,10 +2089,10 @@ public:
 	bool SetColor(int select, DWORD col);
 	void DoPlot(anyOutput *o);
 	void DoMark(anyOutput *o, bool mark);
-	bool Command(int cmd, void *tmpl, anyOutput *o);
-	bool PropertyDlg();
-	void RegGO(void *n);
-	bool FileIO(int rw);
+	virtual bool Command(int cmd, void *tmpl, anyOutput *o);
+	virtual bool PropertyDlg();
+	virtual void RegGO(void *n);
+	virtual bool FileIO(int rw);
 
 	void * ObjThere(int x, int y);
 	void Track(POINT *p, anyOutput *o);
@@ -2050,7 +2107,7 @@ private:
 	long nObs, nmaxObs;
 	DWORD crea_flags;
 	Drag3D *drag;
-	fPOINT3D cub1, cub2, rotC, cu1, cu2, rc;
+	fPOINT3D cu1, cu2, rc;
 	obj_desc **dispObs;
 
 	bool AddPlot(int family);
@@ -2083,6 +2140,28 @@ public:
 	bool PropertyDlg();
 };
 
+class Func3D:public Plot3D {
+public:
+	Func3D(GraphObj *par, DataObj *d);
+	Func3D(int src);
+	~Func3D();
+	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool PropertyDlg();
+	void RegGO(void *n);
+	bool FileIO(int rw);
+
+private:
+	bool Update();
+
+	double x1, x2, xstep, z1, z2, zstep;
+	int g_idx;
+	LineDEF Line;
+	FillDEF Fill;
+	char *param, *cmdxy;
+	DataObj *gda;
+	Grid3D  *gob;
+};
+
 class Graph:public GraphObj{
 public:
 	long NumPlots;
@@ -2188,7 +2267,7 @@ public:
 	char DecPoint[2], ColSep[2];
 	char *svgAttr, *svgScript, *currPath, *IniFile;
 	char *File1, *File2, *File3, *File4, *File5, *File6;
-	double min4log;
+	double min4log, ss_txt;
 	RECT clipRC;
 
 	Default();
@@ -2259,11 +2338,11 @@ public:
 #define UNDO_STORESET 0x1000
 class UndoObj {
 	enum {UNDO_UNDEFINED, UNDO_DEL_GO, UNDO_GOLIST, UNDO_DROPMEM,
-		UNDO_VALDWORD, UNDO_VALINT, UNDO_OBJCONF, UNDO_OBJCONF_1,
-		UNDO_LFP, UNDO_MOVE, UNDO_RECT, UNDO_STRING, UNDO_ROTDEF,
-		UNDO_SETGO, UNDO_LINEDEF, UNDO_FILLDEF, UNDO_AXISDEF, 
-		UNDO_LFP3D, UNDO_FLOAT, UNDO_MEM, UNDO_MUTATE, UNDO_DROPGOLIST,
-		UNDO_TEXTDEF, UNDO_SAVVAR, UNDO_DATA, UNDO_ET};
+		UNDO_VALDWORD, UNDO_VALINT, UNDO_VALLONG, UNDO_OBJCONF, UNDO_OBJCONF_1,
+		UNDO_LFP, UNDO_POINT, UNDO_VOIDPTR, UNDO_MOVE, UNDO_RECT,
+		UNDO_STRING, UNDO_ROTDEF, UNDO_SETGO, UNDO_LINEDEF, UNDO_FILLDEF,
+		UNDO_AXISDEF, UNDO_LFP3D, UNDO_FLOAT, UNDO_MEM, UNDO_MUTATE, 
+		UNDO_DROPGOLIST, UNDO_TEXTDEF, UNDO_SAVVAR, UNDO_DATA, UNDO_ET};
 	typedef struct _UndoInfo {
 		int cmd;
 		DWORD flags;
@@ -2294,6 +2373,8 @@ class UndoObj {
 
 public:
 	int *pcb;
+	anyOutput *cdisp, *ldisp;
+	bool busy;
 
 	UndoObj();
 	~UndoObj();
@@ -2301,7 +2382,7 @@ public:
 	void SetDisp(anyOutput *o);
 	void KillDisp(anyOutput *o);
 	void InvalidGO(GraphObj *go);
-	void Pop();
+	void Pop(anyOutput *o);
 	void Restore(bool redraw, anyOutput *o);
 	void ListGOmoved(GraphObj **oldlist, GraphObj **newlist, long size);
 	void DeleteGO(GraphObj **go, DWORD flags, anyOutput *o);
@@ -2311,7 +2392,10 @@ public:
 	void DropMemory(GraphObj *parent, void **mem, DWORD flags);
 	void SavVarBlock(GraphObj *parent, void **mem, DWORD flags);
 	void ValDword(GraphObj *parent, DWORD *val, DWORD flags);
+	void Point(GraphObj *parent, POINT *pt, anyOutput * o, DWORD flags);
+	void VoidPtr(GraphObj *parent, void **pptr, void *ptr, anyOutput * o, DWORD flags);
 	void ValInt(GraphObj *parent, int *val, DWORD flags);
+	void ValLong(GraphObj *parent, long *val, DWORD flags);
 	void ObjConf(GraphObj *go, DWORD flags);
 	int SaveLFP(GraphObj *go, lfPOINT *lfp, DWORD flags);
 	void MoveObj(GraphObj *go, lfPOINT *lfp, DWORD flags);
@@ -2326,13 +2410,12 @@ public:
 	void ValLFP3D(GraphObj *go, fPOINT3D *lfp, DWORD flags);
 	void ValFloat(GraphObj *parent, double *val, DWORD flags);
 	void DataMem(GraphObj *go, void **mem, int size, long *count, DWORD flags);
-	void DataObject(GraphObj *go, anyOutput *o, DataObj *d, DWORD flags);
+	void DataObject(GraphObj *go, anyOutput *o, DataObj *d, RECT *rc, DWORD flags);
 	void TextCell(EditText *et, anyOutput *o, char *text, int *cur, int *m1, int *m2, void* DaO, DWORD flags);
 
 private:
 	UndoInfo **buff, **buff0;
 	int stub1, ndisp;
-	anyOutput *cdisp, *ldisp;
 	UndoBuff **buffers;
 
 	int NewItem(int cmd, DWORD flags, GraphObj *owner, void *data, void **loc);
@@ -2353,6 +2436,7 @@ char *OpenDataName(char *oldname);
 void InfoBox(char *Msg);
 void ErrorBox(char *Msg);
 bool YesNoBox(char *Msg);
+int YesNoCancelBox(char *Msg);
 void Qt_Box();
 void HideTextCursor();
 void HideTextCursorObj(anyOutput *out);
@@ -2361,6 +2445,8 @@ void HideCopyMark();
 void ShowCopyMark(anyOutput *out, RECT *mrk, int nRec);
 void InitTextCursor(bool init);
 void EmptyClip();
+void CopyText(char *txt, int len);
+unsigned char* PasteText();
 void GetDesktopSize(int *width, int *height);
 void FindBrowser();
 void LoopDlgWnd();
@@ -2386,8 +2472,7 @@ void GetNewFill(FillDEF *oldfill);
 void ShowBanner(bool show);
 void RLPlotInfo();
 bool DoSpShSize(DataObj *dt);
-bool FillSsRange(DataObj *d, char **range);
-bool GetCopyRange(RECT *rc, DataObj *d);
+bool FillSsRange(DataObj *d, char **range, GraphObj *msg_go);
 bool GetBitmapRes(double *res, double *width, double *height, char *header);
 void OD_scheme(int, void *, RECT *, anyOutput *, void *, int);
 FillDEF *GetSchemeFill(int *i);
@@ -2419,6 +2504,7 @@ double NiceValue(double fv);
 char *Int2ColLabel(int nr, bool uc);
 char *str2xml(char *str);
 char **split(char *str, char sep, int *nl);
+char *fit_num_rect(anyOutput *o, int max_width, char *num_str);
 void SetMinMaxRect(RECT *rc, int x1, int y1, int x2, int y2);
 void UpdateMinMaxRect(RECT *rc, int x, int y);
 void IncrementMinMaxRect(RECT *rc, int i);
@@ -2438,10 +2524,12 @@ unsigned long isqr(unsigned long n);
 bool MatMul(double a[3][3], double b[3][3], double c[3][3]);
 char *GetNumFormat(double Magn);
 void DeleteGO(GraphObj *go);
+bool DeleteGOL(GraphObj ***gol, long n, GraphObj *go, anyOutput *o);
 bool BackupFile(char *FileName);
 bool IsRlpFile(char *FileName);
 bool IsXmlFile(char *FileName);
 bool FileExist(char *FileName);
+bool IsPlot3D(GraphObj *g);
 void *memdup(void *ptr, int cb_old, int cb_new);
 double sininv(double val);
 double trig2deg(double si, double csi);
@@ -2474,8 +2562,10 @@ void DoExportTif(GraphObj *g, char *FileName, DWORD flags);
 
 //prototypes mfcalc.cpp
 bool do_xyfunc(DataObj *, double, double, double, char *, lfPOINT **, long *, char *);
+bool do_func3D(DataObj *d, double x1, double x2, double xstep, double z1, double z2, double zstep, 
+	char *expr, char *param);
 anyResult *do_formula(DataObj *, char *);
-bool MoveFormula(DataObj *d, char *of, char *nf, int dx, int dy);
+bool MoveFormula(DataObj *d, char *of, char *nf, int dx, int dy, int r0, int c0);
 int do_fitfunc(DataObj *d, char *rx, char *ry, char *rz, char **par, char *expr, 
 	double conv, int maxiter, double *chi_2);
 
@@ -2485,11 +2575,31 @@ void free_dmatrix(double **m, int nrl, int nrh, int ncl, int);
 bool mrqmin(double *, double *, double *, int, double **, int, int *, int, double **, double **, double *,
 	void (*funcs)(double, double, double **, double *, double *, int), double *);
 bool Check_MRQerror();
+void SortFpArray(int n, lfPOINT *vals);
+void spline(lfPOINT *v, int n, double *y2);
 double gammln(double xx);
 double factrl(int n);
-double t_dist(double t, double df);
+double gammp(double a, double x);
+double gammq(double a, double x);
+double betai(double a, double b, double x);
+double bincof(double n, double k);
+double binomdistf(double k, double n, double p);
+double betaf(double z, double w);
+double errf(double x);
+double errfc(double x);
+double norm_dist(double x, double m, double s);
+double chi_dist(double x, double df, double);
+double t_dist(double t, double df, double);
+double pois_dist(double x, double m, double);
 double f_dist(double f, double df1, double df2);
+double distinv(double (*sf)(double, double, double), double df1, double df2, double p, double x0);
 void d_quartile(int n, double *v, double *q1, double *q2, double *q3);
 double d_amean(int n, double *v);
 double d_gmean(int n, double *v);
 double d_hmean(int n, double *v);
+double d_pearson(double *x, double *y, int n, char *dest, DataObj *data);
+double d_spearman(double *x, double *y, int n, char *dest, DataObj *data);
+double d_regression(double *x, double *y, int n, char *dest, DataObj *data);
+double d_ttest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
+double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
+
